#General purpose data wrangling/visualization packages
import pandas as pd
import seaborn as sns
import numpy as np
import matplotlib
from matplotlib.colors import LogNorm
import matplotlib.pyplot as plt
import os,pickle,sys,shutil
from sklearn.preprocessing import MinMaxScaler
sns.set_context('talk')

#Image processing packages
from matplotlib import image as mplImage
import pytesseract
from PIL import Image
from scipy import ndimage
import cv2

#Clustering
import hdbscan

#Miscellaneous
from itertools import tee
from tqdm.auto import trange
from scipy.signal import argrelmin,find_peaks,savgol_filter
import warnings
warnings.filterwarnings("ignore")

def ranges(nums):
    nums = sorted(set(nums))
    gaps = [[s, e] for s, e in zip(nums, nums[1:]) if s+1 < e]
    edges = iter(nums[:1] + sum(gaps, []) + nums[-1:])
    return list(zip(edges, edges))

def pairwise(iterable):
    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
    a, b = tee(iterable)
    next(b, None)
    return zip(a, b)

def get_blocks(values,cutoff):
    result = []
    for i,val in enumerate(values):
        values2 = [values[x] for x in range(len(values)) if x != i]
        group = [val]
        for val2 in values2:
            if abs(val2 - val) < cutoff:
                group.append(val2)
        result.append(group)
    return result

def np_array_to_hex(array):
    array = np.asarray(array, dtype='uint32')
    array = ((array[:, :, 0]<<16) + (array[:, :, 1]<<8) + array[:, :, 2])
    return array

def img_array_to_single_val(image, color_codes):
    image = image.dot(np.array([65536, 256, 1], dtype='int32'))
    result = np.ndarray(shape=image.shape, dtype=int)
    result[:,:] = -1
    for rgb, idx in color_codes.items():
        rgb = rgb[0] * 65536 + rgb[1] * 256 + rgb[2]
        result[image==rgb] = idx
    return result

def transparent_cmap(cmap, N=255):
    "Copy colormap and set alpha values"

    mycmap = cmap
    mycmap._init()
    #mycmap._lut[:,-1] = np.linspace(0, 0.8, N+4)
    mycmap._lut[0,-1] = 0
    return mycmap

def returnColorScale(colorBar):    
    rgbaList = np.dsplit(colorBar,colorBar.shape[2])
    colorDfList = []
    for d in rgbaList:
        colorDf = pd.DataFrame(d[:,:,0],index=list(range(colorBar.shape[0])),columns=list(range(colorBar.shape[1])))
        colorDf.index.name = 'Row'
        colorDf.columns.name = 'Column'
        colorDfList.append(colorDf)
    colorDf = pd.concat(colorDfList,keys=['R','G','B','A'],names=['Color'])
    colorScale = colorDf.loc[['R','G','B']].iloc[:,int(colorDf.shape[1]/2)].unstack('Color').values.tolist()[::-1] + [[1,0,0]]

    rgbColorScale = []
    for elem in colorScale:
        elem = [int(255*x) for x in elem]
        rgbColorScale.append(elem)
    
    trueRGBColorScale = []
    offset = 1
    for r in range(rgbColorScale[0][0]+offset,-1,-1):
        trueRGBColorScale.append([r,0,255])
    for g in range(1,256):
        trueRGBColorScale.append([0,g,255])
    for b in range(255,-1,-1):
        trueRGBColorScale.append([0,255,b])
    for r in range(1,256):
        trueRGBColorScale.append([r,255,0])
    for g in range(255,-1,-1):
        trueRGBColorScale.append([255,g,0])

    return trueRGBColorScale

def returnColorScaleSpan(legend,colorScale,colorBarScale):
    croppedLegend = np.multiply(legend[60:105,:75],255)
    pIm = Image.fromarray(np.uint8(croppedLegend))
    fullString = pytesseract.image_to_string(pIm)
    splitStrings = fullString.split('\n')
    for splitString in splitStrings:
        if 'Min' in splitString or 'Max' in splitString:
          #Account for occasinoal pytesseract failures
          if ' = ' in splitString:
            splitChar = ' = '
          else:
            if ' =' in splitString:
              splitChar = ' ='
            elif '= ' in splitString:
              splitChar = '= '
            else:
              if '=' in splitString:
                splitChar = '='
              else:
                print('PyTessearct error; equal sign not found. Split string is:'+splitString)
          if 'Min' in splitString:
              number = splitString.split(splitChar)[1]
              minScalingFactor = 10**int(number[-1])
              scaleStart = float(number[:4])*minScalingFactor
          elif 'Max' in splitString:
              number = splitString.split(splitChar)[1]
              maxScalingFactor = 10**int(number[-1])
              scaleEnd = float(number[:4])*maxScalingFactor
            
    linearScale = np.linspace(scaleStart,scaleEnd,num=len(colorScale))
    return linearScale,scaleStart

def returnLuminescentImageComponents(luminescent,visualize=False):
    occupancyCutoff = 300
    headerCutoff = 50

    rgbSample = luminescent[:,:,:]
    #Separate colorbar from mouse samples
    bwMatrix = np.zeros(luminescent.shape)[:,:,:]
    for row in range(luminescent.shape[0]):
        for col in range(luminescent.shape[1]):
            pixel = luminescent[row,col].tolist()[:3]
            if pixel != [1,1,1]:
                bwMatrix[row,col] = 1

    #Split horizontally
    componentSeparationDfHorizontal = pd.DataFrame(bwMatrix[:,:,0].sum(axis=1),index=list(range(bwMatrix.shape[0])),columns=['Sample Pixel Number'])
    componentSeparationDfHorizontal.index.name = 'Row'
    rowBreakpoints = []
    for row in range(1,componentSeparationDfHorizontal.shape[0]-1):
        pixelNum = componentSeparationDfHorizontal.iloc[row,0]
        if (componentSeparationDfHorizontal.iloc[row-1,0] < occupancyCutoff and componentSeparationDfHorizontal.iloc[row,0] > occupancyCutoff) or (componentSeparationDfHorizontal.iloc[row,0] > occupancyCutoff and componentSeparationDfHorizontal.iloc[row+1,0] < occupancyCutoff):
            rowBreakpoints.append(row)

    #Split vertically
    componentSeparationDf = pd.DataFrame(bwMatrix[:,:,0].sum(axis=0),index=list(range(bwMatrix.shape[1])),columns=['Sample Pixel Number'])
    componentSeparationDf.index.name = 'Column'
    columnBreakpoints = []
    for row in range(1,componentSeparationDf.shape[0]-1):
        pixelNum = componentSeparationDf.iloc[row,0]
        if (componentSeparationDf.iloc[row-1,0] < occupancyCutoff and componentSeparationDf.iloc[row,0] > occupancyCutoff) or (componentSeparationDf.iloc[row,0] > occupancyCutoff and componentSeparationDf.iloc[row+1,0] < occupancyCutoff):
            columnBreakpoints.append(row)
    
    #Split off colorbar
    rgbScale = luminescent[:,columnBreakpoints[2]:columnBreakpoints[3]+1,:]
    colorBarRowIndices,colorBarColumnIndices = [],[]
    for i in range(rgbScale.shape[0]): # for every pixel:
        for j in range(rgbScale.shape[1]):
            pixel = rgbScale[i,j].tolist()[:3]
            if len(np.unique(pixel)) == 1 and np.unique(pixel) in [0,1]:
                pass
            else:
                colorBarRowIndices.append(i)
                colorBarColumnIndices.append(j)
    
    if rowBreakpoints[0] > headerCutoff:
        interval = 596
    else:
        interval = 706
    if rowBreakpoints[-1] - rowBreakpoints[-2] != interval:
      #print(rowBreakpoints)
      rowBreakpoints[-1] = rowBreakpoints[-2]+interval
    if columnBreakpoints[1] - columnBreakpoints[0] != interval:
      #print(columnBreakpoints)
      columnBreakpoints[1] = columnBreakpoints[0]+interval
    colorBar = rgbScale[min(colorBarRowIndices):max(colorBarRowIndices)+1,min(colorBarColumnIndices):max(colorBarColumnIndices)+1,:]
    miceSamples = luminescent[rowBreakpoints[-2]:rowBreakpoints[-1],columnBreakpoints[0]:columnBreakpoints[1],:]
    legend = luminescent[max(colorBarRowIndices):,columnBreakpoints[2]:]
    colorBarScale = luminescent[min(colorBarRowIndices):max(colorBarRowIndices)+1,columnBreakpoints[2]+max(colorBarColumnIndices)+1:,:]
    
    if visualize:
        sns.relplot(data=componentSeparationDfHorizontal,x='Row',y='Sample Pixel Number',kind='line',aspect=luminescent.shape[1]/luminescent.shape[0])
        sns.relplot(data=componentSeparationDf,x='Column',y='Sample Pixel Number',kind='line',aspect=luminescent.shape[1]/luminescent.shape[0])
    
    return miceSamples,colorBar,legend,colorBarScale

def findBrightfieldCutoff(brightfield,visualize=False):

    brightfieldDf = pd.DataFrame(brightfield)
    brightfieldDf.index.name = 'Row'
    brightfieldDf.columns.name = 'Column'
    plottingDf = brightfieldDf.stack().to_frame('Intensity')

    visualBrightfieldMatrix = MinMaxScaler(feature_range=(11000,65000)).fit_transform(np.clip(plottingDf.values,a_min=0,a_max=5500))

    hist,bins = np.histogram(plottingDf.values,bins='auto', density=True)
    hist = savgol_filter(hist,7,2)
    maxHist = np.argmax(hist)
    elbow = maxHist+1000
    if visualize:
        g = sns.displot(data=plottingDf,x='Intensity',element='poly')
        g.axes.flat[0].axvline(x=elbow,linestyle=':',color='k')
        blackpointDf = plottingDf.copy()
        blackpointDf.loc[:,:] = visualBrightfieldMatrix
        blackpointDf2 = plottingDf.copy()
        g = sns.displot(data=pd.concat([blackpointDf,blackpointDf2],keys=['Yes','No'],names=['Scaled']),hue='Scaled',kind='hist',x='Intensity',element='poly',fill=False)
        g.set(yscale='log')

    return elbow,visualBrightfieldMatrix.reshape(brightfield.shape)

def rescaleBrightfieldImage(brightfield,brightfield2,luminescentSamples,visualize=False):
    
    rescaledBrightfieldMatrix = cv2.resize(brightfield, dsize=luminescentSamples.shape[:2])
    
    if np.mean(rescaledBrightfieldMatrix) > 10000:
        brightfieldCutoff = 90
        visualBrightfieldMatrix = cv2.resize(brightfield2, dsize=luminescentSamples.shape[:2])
        mouseBrightfieldMatrix = (visualBrightfieldMatrix[:,:,0] > brightfieldCutoff).astype(int)
    else:        
        brightfieldCutoff,visualBrightfieldMatrix = findBrightfieldCutoff(rescaledBrightfieldMatrix,visualize=visualize)
        mouseBrightfieldMatrix = (rescaledBrightfieldMatrix > brightfieldCutoff).astype(int)
    
    if visualize:
        fig = plt.figure()
        sns.heatmap(mouseBrightfieldMatrix,cmap='Greys')
    
    return mouseBrightfieldMatrix,visualBrightfieldMatrix

def horizontallySeparateMice(brightfieldSamples,visualize=False):
    brightfieldDf = pd.DataFrame(brightfieldSamples,index=list(range(brightfieldSamples.shape[0])),columns=list(range(brightfieldSamples.shape[1])))
    brightfieldDf.index.name = 'Row'
    brightfieldDf.columns.name = 'Column'
    brightfieldDf = brightfieldDf.sum(axis=1).to_frame('Count')
    brightfieldDf.loc[:,:] = MinMaxScaler().fit_transform(brightfieldDf.values)

    maxPointRangeCutoff = 0.2
    rangeBreakpoints = []
    for row in range(brightfieldDf.shape[0]-1,-1,-1):
        if brightfieldDf.iloc[row,0] > maxPointRangeCutoff:
            rangeBreakpoints.append(row)
            break        
    for row in range(rangeBreakpoints[0]-1,-1,-1):
        if brightfieldDf.iloc[row,0] < maxPointRangeCutoff:
            rangeBreakpoints.append(row)
            break        
    
    maxPoint = np.argmax(brightfieldDf.iloc[rangeBreakpoints[1]:rangeBreakpoints[0],0]) + rangeBreakpoints[1]
    breakpoints = [0,brightfieldDf.shape[0]-1]

    cutOff = maxPointRangeCutoff/2
    for row in range(maxPoint,-1,-1):
        if brightfieldDf.iloc[row,0] < cutOff:
            breakpoints[0] = row
            break        
    for row in range(maxPoint,brightfieldDf.shape[0]):
        if brightfieldDf.iloc[row,0] < 0.01:
            breakpoints[1] = row
            break        

    if visualize:
        g = sns.relplot(data=brightfieldDf,x='Row',y='Count',kind='line')
        g.axes.flat[0].axhline(y=maxPointRangeCutoff,linestyle=':',color='k')
        g.axes.flat[0].axhline(y=cutOff,linestyle=':',color='r')
        for breakpoint in breakpoints:
            g.axes.flat[0].axvline(x=breakpoint,linestyle=':',color='k')
        g.axes.flat[0].plot(maxPoint, brightfieldDf.iloc[maxPoint,0],color='k',marker='o')

    return breakpoints

def verticallySeparateMice(mouseBrightfieldMatrix,breakpoints,visualize=False):
    
    croppedBrightfieldMatrix = mouseBrightfieldMatrix[breakpoints[0]:breakpoints[1]+1,:]
    verticalMouseSeparationDf = pd.DataFrame(croppedBrightfieldMatrix,index=list(range(croppedBrightfieldMatrix.shape[0])),columns=list(range(croppedBrightfieldMatrix.shape[1])))
    verticalMouseSeparationDf.index.name = 'Row'
    verticalMouseSeparationDf.columns.name = 'Column'
    verticalMouseSeparationDf = verticalMouseSeparationDf.sum(axis=0).to_frame('Count').rolling(window=25).mean().fillna(value=0)
    verticalMouseSeparationDf.loc[:,:] = MinMaxScaler().fit_transform(verticalMouseSeparationDf.values)

    data = savgol_filter(verticalMouseSeparationDf.values.flatten(),31,2)
    data = abs(data)
    mins, _ = find_peaks(-1*data)
    #Further selection; mice have to be a certain distance apart
    trueMins = []
    minMouseWidth = 90
    minGroups = get_blocks(mins,minMouseWidth)
    for group in minGroups:
        if len(group) == 1:
            minIndex = group[0]
        else:
            dataVals = [data[x] for x in group]
            #minIndex = sorted(group)[-1]
            minIndex = group[dataVals.index(min(dataVals))]
        trueMins.append(minIndex)
    trueMins.append(verticalMouseSeparationDf.shape[0]-1)
    trueMins = np.unique(trueMins).tolist()
    #Even further selection; each interval must have a max of at least 15% the overall max
    splitMice = []
    maxCutoff = 0.2
    keptIntervals = []
    trueMins2 = []
    for i,trueMin in enumerate(trueMins[:-1]):
        maxI = 10
        if trueMin+maxI > verticalMouseSeparationDf.shape[0]-1:
            maxI = verticalMouseSeparationDf.shape[0]-trueMin-1
        if data[trueMin+maxI] >= data[trueMin]:
            trueMins2.append(trueMin)
    if data[trueMins[-1]-10] >= data[trueMins[-1]]:
        trueMins2.append(trueMins[-1])
    trueMins = trueMins2
    trueCutoff = min(data) + maxCutoff * (max(data)-min(data))
    for interval in pairwise(trueMins):
        if max(data[interval[0]:interval[1]]) > trueCutoff and abs(interval[1] - interval[0]) >= minMouseWidth:
            keptIntervals.append(interval)

    peaks,finalKeptIntervals = [],[]
    secondaryCutoff = 0.1
    for interval in keptIntervals:
        startpoint = interval[0]
        endpoint = interval[1]
        intervalValues = verticalMouseSeparationDf.query("Column > @startpoint and Column < @endpoint").reset_index()
        peakPoint = np.argmax(intervalValues['Count'].values)
        peak = intervalValues['Column'][peakPoint]
        peaks.append(peak)
        #Even further selection; crop sides of interval at cutoff point; starting from max point
        leftEndpoint = interval[0]
        rightEndpoint = interval[1]
        for val in range(peak,rightEndpoint):
            if intervalValues.query("Column == @val")['Count'].values[0] < secondaryCutoff:
                rightEndpoint = val
                break
        for val in range(peak,leftEndpoint,-1):
            if intervalValues.query("Column == @val")['Count'].values[0] < secondaryCutoff:
                leftEndpoint = val
                break
        finalKeptIntervals.append([leftEndpoint,rightEndpoint])
    
    if visualize:
        verticalMouseSeparationDf.loc[:,:] = data.reshape(-1,1)
        g = sns.relplot(data=verticalMouseSeparationDf,x='Column',y='Count',kind='line')
        g.axes.flat[0].axhline(y=trueCutoff,linestyle=':',color='k')
        g.axes.flat[0].axhline(y=secondaryCutoff,linestyle=':',color='r')
        for interval in finalKeptIntervals:
            g.axes.flat[0].axvline(x=interval[0],linestyle=':',color='k')
            g.axes.flat[0].axvline(x=interval[1],linestyle=':',color='k')
        
    return finalKeptIntervals,peaks

def fullySeparateMice(luminescentSamples,brightfieldSamples,originalBrightfieldSamples,verticalBreakpoints,horizontalBreakpoints,visualize=False):
    
    miceSamples = np.dstack([luminescentSamples,brightfieldSamples])
    splitMice,splitBrightfields = [],[]
    posthocCutoff = 10
    for interval in verticalBreakpoints:
        splitMouse = miceSamples[horizontalBreakpoints[0]:horizontalBreakpoints[1],max(interval[0]-posthocCutoff,0):interval[1]-posthocCutoff,:]
        if len(originalBrightfieldSamples.shape) > 2:
            splitBrightfield = originalBrightfieldSamples[horizontalBreakpoints[0]:horizontalBreakpoints[1],max(interval[0]-posthocCutoff,0):interval[1]-posthocCutoff,:]
        else:
            splitBrightfield = originalBrightfieldSamples[horizontalBreakpoints[0]:horizontalBreakpoints[1],max(interval[0]-posthocCutoff,0):interval[1]-posthocCutoff]            
        splitMice.append(splitMouse)
        splitBrightfields.append(splitBrightfield)
    if visualize:
        fig, axes = plt.subplots(2,len(splitMice),figsize=(2.5*len(splitMice),5))
        for i,sample in enumerate(splitMice):
            if len(splitMice) > 1:
                axes[0,i].set_title(str(i+1))
                axes[0,i].imshow(sample[:,:,:3])
                axes[1,i].imshow(sample[:,:,3],cmap='Greys')
            else:
                axes[0].set_title(str(i+1))
                axes[0].imshow(sample[:,:,:3])
                axes[1].imshow(sample[:,:,3],cmap='Greys')
            if i == 0:
                if len(splitMice) > 1:
                    for ax in [axes[0,i],axes[1,i]]:
                        ax.set_xlabel('')
                        ax.set_xticks([])
                        ax.set_yticks([])
                        ax.spines['top'].set_visible(False)
                        ax.spines['right'].set_visible(False)
                        ax.spines['bottom'].set_visible(False)
                        ax.spines['left'].set_visible(False)                
                    axes[0,i].set_ylabel('Luminescent')
                    axes[1,i].set_ylabel('Brightfield')
                else:
                    for ax in [axes[0],axes[1]]:
                        ax.set_xlabel('')
                        ax.set_xticks([])
                        ax.set_yticks([])
                        ax.spines['top'].set_visible(False)
                        ax.spines['right'].set_visible(False)
                        ax.spines['bottom'].set_visible(False)
                        ax.spines['left'].set_visible(False)                
                    axes[0].set_ylabel('Luminescent')
                    axes[1].set_ylabel('Brightfield')
            else:
                if len(splitMice) > 1:
                    axes[0,i].axis('off')
                    axes[1,i].axis('off')
                else:
                    axes[0].axis('off')
                    axes[1].axis('off')

        fig.tight_layout()

    return splitMice,splitBrightfields

def returnRadianceMetrics(imageTitle,samples,splitBrightfields,colorScale,linearScale,trueMin,sampleNames=[],save_pixel_df=False,visualize=False):            
    outputDir = 'outputData/'
    statisticList = []
    if len(sampleNames) == 0:
        sampleNames = list(map(str,list(range(1,len(samples)+1))))

    radianceColorDict,pixelIntensityColorDict = {},{}
    for i,color in enumerate(colorScale):
        radianceColorDict[tuple(color)] = linearScale[i]
        pixelIntensityColorDict[tuple(color)] = i
    pixelDfList,pixelMatrixList,pixelMatrixNameList, = [],[],[]
    
    for sn,rgbSample in enumerate(samples):
        #Convert to RGB
        trueRGBSample = np.multiply(255,rgbSample[:,:,:3]).astype(int)
        #Create pixelwise radiance dataframe
        if visualize or save_pixel_df:
          intensityMatrix = img_array_to_single_val(trueRGBSample, pixelIntensityColorDict)
          radianceMatrix = img_array_to_single_val(trueRGBSample, radianceColorDict)
          brightfieldMatrix = rgbSample[:,:,3]
          if visualize:
              pixelIntensityDf = pd.DataFrame(intensityMatrix)
              pixelIntensityDf.index.name = 'Row'
              pixelIntensityDf.columns.name = 'Column'
              pixelRadianceDf = pd.DataFrame(radianceMatrix)
              pixelRadianceDf.index.name = 'Row'
              pixelRadianceDf.columns.name = 'Column'
              pixelBrightfieldDf = pd.DataFrame()
              pixelBrightfieldDf.index.name = 'Row'
              pixelBrightfieldDf.columns.name = 'Column'
              pixelRadianceAndBrightfieldDf = pd.concat([pixelIntensityDf.stack(),pixelRadianceDf.stack(),pixelBrightfieldDf.stack()],axis=1,keys=['Intensity','Radiance','Brightfield'])
              pixelDfList.append(pixelRadianceAndBrightfieldDf)
          else:
           imageMatrixPath = outputDir+'imageMatrices/'
           np.save(imageMatrixPath+'-'.join([imageTitle,sampleNames[sn]]),np.dstack([radianceMatrix,brightfieldMatrix,splitBrightfields[sn]]))
           with open(imageMatrixPath+'-'.join([imageTitle,sampleNames[sn]])+'.pkl','wb') as f:
               pickle.dump([trueMin,linearScale[-1]],f)

#             pixelMatrixList.append(np.dstack([radianceMatrix,brightfieldMatrix,splitBrightfields[sn]]))
#             pixelMatrixNameList.append('-'.join([imageTitle,sampleNames[sn]]))

        #Remove non mouse pixels
        brightfieldMask = rgbSample[:,:,3] == 1
        trueRGBSample = trueRGBSample[brightfieldMask]
        
        #Convert pixel intensity arrays to radiance arrays
        trueRGBSample = trueRGBSample[:,np.newaxis,:]
        scoringMatrix = img_array_to_single_val(trueRGBSample, pixelIntensityColorDict)
        radianceMatrix = img_array_to_single_val(trueRGBSample, radianceColorDict)

        #Selects non color scale pixels
        colorScaleBoolean = (radianceMatrix == -1).astype(int)
        additionMatrix = np.multiply(colorScaleBoolean,0) + colorScaleBoolean
                
        #Set mouse pixels (white/greyscale) to minimum detectable colorbar value
        radianceMatrix = np.add(radianceMatrix,additionMatrix)
        
        #Image statistics
        avgPixelIntensity = np.sum(scoringMatrix+1)/(scoringMatrix.shape[0])
        numTumorPixels = radianceMatrix[radianceMatrix != 0].shape[0]
        
        #Radiance statistics
        avgRadiance = np.sum(radianceMatrix)/(trueRGBSample.shape[0])
        totalRadiance = np.sum(radianceMatrix)
        
        #Tumor spatial area statistics
        tumorAreaCovered = numTumorPixels/trueRGBSample.shape[0]
        
        statisticList.append([avgPixelIntensity,avgRadiance,totalRadiance,tumorAreaCovered])

    if visualize:
      pixelDf = pd.concat(pixelDfList,keys=sampleNames,names=['Sample'])
      if 'processedImages' not in os.listdir(outputDir):
        os.mkdir(outputDir+'processedImages')
    else:
      pixelDf = []
    statisticMatrix = np.matrix(statisticList)
    statisticDf = pd.DataFrame(statisticMatrix,index=sampleNames,columns=['Average Pixel Intensity','Average Radiance','Total Radiance','Tumor Fraction'])
    statisticDf.index.name = 'Sample'

    if visualize:
      intensityDf = pixelDf['Radiance'].unstack('Column')
      brightfieldDf = pixelDf['Brightfield'].unstack('Column')
      initialDf = np.add((intensityDf.values == -1).astype(int),(brightfieldDf.values == 0).astype(int))
      initialDf = (initialDf != 0).astype(int) 
      additionDf = np.multiply(initialDf,trueMin-1)
      intensityDf.iloc[:,:] = np.multiply(intensityDf.values,1-initialDf) + additionDf
      numSamples = len(intensityDf.index.unique('Sample'))
      fig, axes = plt.subplots(1,numSamples,figsize=(2.5*numSamples,5))
      fig.subplots_adjust(right=0.8)
      cbar_ax = fig.add_axes([0.85, 0.15, 0.02, 0.7])
      for i,sample in enumerate(intensityDf.index.unique('Sample')):
        b = splitBrightfields[i]
        sampleDf = intensityDf.query("Sample == @sample").dropna(axis=1) 
        cmap = sns.color_palette("magma", as_cmap=True)
        sampleDf.iloc[-1,-1] = np.nanmax(linearScale[-1])
        if len(intensityDf.index.unique('Sample')) > 1:
            g = sns.heatmap(sampleDf,cmap=transparent_cmap(cmap),cbar= i == 0,ax=axes[i],norm=matplotlib.colors.LogNorm(),vmin=np.nanmin(intensityDf.values),vmax=linearScale[-1],cbar_ax=cbar_ax,cbar_kws={'label':'Radiance\n(p/sec/cm$^2$/sr)'})
            axes[i].set_title('Sample '+str(i+1))
            axes[i].imshow(b,zorder=0, cmap='gray')
            axes[i].axis('off')
        else:
            g = sns.heatmap(sampleDf,cmap=transparent_cmap(cmap),cbar= i == 0,ax=axes,norm=matplotlib.colors.LogNorm(),vmin=np.nanmin(intensityDf.values),vmax=linearScale[-1],cbar_ax=cbar_ax,cbar_kws={'label':'Radiance\n(p/sec/cm$^2$/sr)'})
            axes.set_title('Sample '+str(i+1))
            axes.imshow(b,zorder=0, cmap='gray')
            axes.axis('off')
            
            
      fig.suptitle(imageTitle)
      fig.subplots_adjust(top=0.85)
      fig.savefig(outputDir+'processedImages'+imageTitle+'.png',bbox_inches='tight')
      plt.close()
    
    return statisticDf

def processGroupedMouseImage(imageTitle,luminescent,brightfield,brightfield2,visualize=False,save_pixel_df=False,sampleNames = [],save_images=False):
    
    #Preprocess images
    luminescentSamples,colorBar,legend,colorBarScale = returnLuminescentImageComponents(luminescent,visualize=visualize)
    
    colorScale = returnColorScale(colorBar)
    linearScale,trueMin = returnColorScaleSpan(legend,colorScale,colorBarScale)
    brightfieldSamples,originalBrightfieldSamples = rescaleBrightfieldImage(brightfield,brightfield2,luminescentSamples,visualize=visualize)

    #Crop images
    horizontalBreakpoints = horizontallySeparateMice(brightfieldSamples,visualize=visualize)
    verticalBreakpoints,peaks = verticallySeparateMice(brightfieldSamples,horizontalBreakpoints,visualize=visualize)    
    splitMice,splitBrightfields = fullySeparateMice(luminescentSamples,brightfieldSamples,originalBrightfieldSamples,verticalBreakpoints,horizontalBreakpoints,visualize=visualize)
    
    #Create radiance statistic dataframe and pixel-wise radiance dataframe
    radianceStatisticDf = returnRadianceMetrics(imageTitle,splitMice,splitBrightfields,colorScale,linearScale,trueMin,sampleNames = sampleNames,save_pixel_df=save_pixel_df,visualize=save_images)
    return radianceStatisticDf,peaks

def addTrueIndexToDataframe(radianceStatisticDf,sampleNameFile):
    matrixList,tupleList = [],[]
    for row in range(sampleNameFile.shape[0]):
        sampleName = list(sampleNameFile['Group'])[row]
        time = list(sampleNameFile['Day'])[row]
        sampleStatistics = radianceStatisticDf.xs([sampleName,time],level=['Group','Day'])
        matrixList.append(sampleStatistics.values)
        for sample in sampleStatistics.index.unique('Sample'):
          tupleList.append(sampleNameFile.iloc[row,:].values.tolist()+[sample])

    fullMatrix = np.vstack(matrixList)
    multiIndex = pd.MultiIndex.from_tuples(tupleList,names=list(sampleNameFile.columns)+['Sample']).droplevel('Group')
    completeDf = pd.DataFrame(fullMatrix,index=multiIndex,columns=radianceStatisticDf.columns)
    if 'SampleNames' in completeDf.index.names:
        completeDf = completeDf.droplevel('SampleNames')
    return completeDf

def addTrueIndexToPixelDataframe(radiancePixelDf,sampleNameFile):
    matrixList,tupleList = [],[]
    for row in range(sampleNameFile.shape[0]):
        sampleName = list(sampleNameFile['Group'])[row]
        time = list(sampleNameFile['Day'])[row]
        sampleStatistics = radiancePixelDf.xs([sampleName,time],level=['Group','Day'])
        matrixList.append(sampleStatistics.values)
        for sample in sampleStatistics.index.unique('Sample'):
          sampleDf = sampleStatistics.query("Sample == @sample").reset_index()
          for i in range(sampleDf.shape[0]):
            tupleList.append(sampleNameFile.iloc[row,:].values.tolist()+[sample,sampleDf['Row'][i],sampleDf['Column'][i]])

    fullMatrix = np.vstack(matrixList)
    multiIndex = pd.MultiIndex.from_tuples(tupleList,names=list(sampleNameFile.columns)+['Sample','Row','Column']).droplevel('Group')
    completeDf = pd.DataFrame(fullMatrix,index=multiIndex,columns=radiancePixelDf.columns)
    if 'SampleNames' in completeDf.index.names:
        completeDf = completeDf.droplevel('SampleNames')
    return completeDf

def luminescentBrightfieldMatchCheck(sampleNameFile,save_pixel_df=False):
  
  inputDir = 'inputData/'
  outputDir = 'outputData/'
  print(sampleNameFile)
  days = [x for x in pd.unique(sampleNameFile['Day'])]

  unmatchedGroups = []
  if save_pixel_df:
    if 'imageMatrices' not in os.listdir(outputDir):
      os.mkdir(outputDir+'imageMatrices')    
    
  for i in range(len(days)):
      day = days[i]
      tempDf = sampleNameFile.query("Day == @day")
      groups = list(pd.unique([x for x in pd.unique(tempDf['Group'])]))
      luminescentImages = [x.split('.')[0][0] for x in os.listdir(inputDir+'luminescent/'+day+'/') if '.DS' not in x]
      brightfieldImages = [x.split('.')[0][0] for x in os.listdir(inputDir+'brightfield/'+day+'/') if '.DS' not in x]
      for j in range(len(groups)):
          group = groups[j]
          if group not in luminescentImages or group not in brightfieldImages:
            if group not in luminescentImages:
              unmatchedGroups.append('luminescent/'+day+'/'+group)
            else:
              unmatchedGroups.append('brightfield/'+day+'/'+group)
  return unmatchedGroups

def amendSampleNames(fullDf,allPeaks,sampleNameFile,fullSplitGroupDict,save_pixel_df=False):
    base_dir = 'outputData/'

    clusterer = hdbscan.HDBSCAN(min_cluster_size=int(len(allPeaks)*0.1))
    cluster_labels = clusterer.fit_predict(np.array(allPeaks).reshape(-1,1))
    cluster_labels = [str(x+1) for x in cluster_labels]  
    tempDf = pd.DataFrame({'Max':allPeaks,'Cluster':cluster_labels})
    sortedClusterList = tempDf.groupby(['Cluster']).mean().sort_values(by='Max').index.get_level_values('Cluster').tolist()
    
    topX = 5
    topXclusters = tempDf.groupby(['Cluster']).count().sort_values(by=['Max'],ascending=False).index.get_level_values('Cluster').tolist()[:topX]    
    allClusterMeansDf = tempDf.groupby(['Cluster']).mean()
    clusterMeans = sorted([allClusterMeansDf.loc[x].values[0] for x in topXclusters])
    partitions = [0] + [(clusterMeans[x]+clusterMeans[x+1])/2 for x in range(len(clusterMeans)-1)] + [np.max(tempDf['Max'])]
    new_cluster_labels = []
    for row in range(tempDf.shape[0]):
        val = tempDf['Max'][row]
        for i in range(len(partitions)-1):
            if val > partitions[i] and val <= partitions[i+1]:
                new_cluster_labels.append(str(i+1))
                break
    
    #new_cluster_labels = [str(sortedClusterList.index(x)+1) for x in cluster_labels]
    
    newFullDf = fullDf.copy()
    newFullDf.index.names = ['Position' if x == 'Sample' else x for x in fullDf.index.names]
    newFullDf = newFullDf.assign(Sample=new_cluster_labels).set_index('Sample', append=True)
    
    fullDfList = []
    splitGroups = [',,'.join(x.split(',,')[:2]) for x in fullSplitGroupDict]
    matrixRenamingDict = {}
    for currentDay in newFullDf.index.unique('Day'):
        dayDf = newFullDf.query("Day == @currentDay")
        for currentGroup in dayDf.index.unique('Group'):
            indexingKey = ',,'.join([currentDay,currentGroup])
            if indexingKey in splitGroups:
                keys = []
                for index,elem in enumerate(splitGroups):
                    if elem == indexingKey:
                        keys.append(list(fullSplitGroupDict.keys())[index])
                if type(keys) == str:
                    keys = [keys]
                subsetDfList = []
                splitPositionDict = {}
                for key in keys:
                    splitPositionDict[key.split(',,')[2]] = 0
                for key in keys:
                    splitKey = key.split(',,')
                    day,group,samples = splitKey[0],splitKey[1],splitKey[2].split(',')
                    renamingPositions = fullSplitGroupDict[key]
                    subsetDf = newFullDf.query("Day == @day and Group == @group and Position == @samples").query("Sample == @renamingPositions")
                    for position,renamedPosition in zip(subsetDf.index.unique('Position'),renamingPositions):
                        splitPositionDict[splitKey[2]]+=1
                        trueIndex = newFullDf.query("Day == @day and Group == @group and Position == @samples").index.unique('Sample').tolist().index(renamedPosition)
                        matrixRenamingDict['-'.join([currentDay,currentGroup,renamedPosition])] = '-'.join([currentDay,currentGroup+','.join(renamingPositions),str(trueIndex+1)])
                    subsetDfList.append(subsetDf)
                renamedEntry = pd.concat(subsetDfList).sort_values(by=['Sample'])
            else:
                renamedEntry = newFullDf.query("Day == @currentDay and Group == @currentGroup")
                for position,renamedPosition in zip(renamedEntry.index.unique('Position'),renamedEntry.index.unique('Sample')):
                    matrixRenamingDict['-'.join([currentDay,currentGroup,renamedPosition])] = '-'.join([currentDay,currentGroup,position])
            if 'SampleNames' in sampleNameFile.columns:
                sampleNameVal = sampleNameFile[(sampleNameFile["Day"] == currentDay) & (sampleNameFile["Group"] == currentGroup)]['SampleNames'].values[0]
                if not pd.isna(sampleNameVal) and sampleNameVal != '':
                    renamingDict = {}
                    for oldSampleName,newSampleName in zip(renamedEntry.index.unique('Sample').tolist(),sampleNameVal.split(',')):
                        renamingDict[oldSampleName] = newSampleName
                        oldSampleKey = matrixRenamingDict.pop('-'.join([currentDay,currentGroup,oldSampleName]))
                        matrixRenamingDict['-'.join([currentDay,currentGroup,newSampleName])] = oldSampleKey
                    renamedEntry = renamedEntry.rename(renamingDict,level='Sample')
            renamedEntry = renamedEntry.sort_values(by=['Sample']).droplevel('Position')
            fullDfList.append(renamedEntry)
    
    if save_pixel_df:
        savezDict,minScaleDict = {},{}
        for savezKey in matrixRenamingDict:
            oldFileName = matrixRenamingDict[savezKey]
            savezDict[savezKey] = np.load(base_dir+'imageMatrices/'+oldFileName+'.npy')            
            minScaleDict[savezKey] = pickle.load(open(base_dir+'imageMatrices/'+oldFileName+'.pkl','rb'))
        
        #Save concatenated files
        experimentName = os.getcwd().split('/')[-1]
        np.savez_compressed(base_dir+experimentName+'-pixel',**savezDict)
        with open(base_dir+experimentName+'-minScale.pkl','wb') as f:
            pickle.dump(minScaleDict,f)
        #Delete temporary directory
        shutil.rmtree(base_dir+'imageMatrices/')
        
    fullDf = pd.concat(fullDfList)    
    return fullDf

def checkSplitGroups(day,group,allPeaks,sampleNames=[],visualize=False,save_df=True,save_pixel_df=False,save_images=False):
    base_dir = 'inputData/'
    luminescentImages = sorted([x.split('.')[0] for x in os.listdir(base_dir+'luminescent/'+day+'/') if len(x.split('.')[0]) > 1 and group in x.split('.')[0]])
    brightfieldImages = sorted([x.split('.')[0] for x in os.listdir(base_dir+'brightfield/'+day+'/') if len(x.split('.')[0]) > 1 and group in x.split('.')[0]])
    splitGroupDict = {}
    if len(luminescentImages) > 1:
        if set(luminescentImages) == set(brightfieldImages):
            groupDfList,groupPixelDfList,groupPeaksList = [],[],[]
            splitIndex = 0
            for splitLuminescentFileName in luminescentImages:
                #Read in luminescent image
                if splitLuminescentFileName+'.png' in os.listdir(base_dir+'luminescent/'+day):
                  fileName = base_dir+'luminescent/'+day+'/'+splitLuminescentFileName+'.png'
                else:
                  fileName = base_dir+'luminescent/'+day+'/'+splitLuminescentFileName+'.PNG'
                luminescent = mplImage.imread(fileName)[:,:,:3]

                #Read in brightfield image
                if splitLuminescentFileName+'.tif' in os.listdir(base_dir+'brightfield/'+day):
                  fileName = base_dir+'brightfield/'+day+'/'+splitLuminescentFileName+'.tif'
                else:
                  fileName = base_dir+'brightfield/'+day+'/'+splitLuminescentFileName+'.TIF'
                brightfield = mplImage.imread(fileName)
                brightfield2 = cv2.imread(fileName)        
                positionsToKeep = splitLuminescentFileName[1:].split(',')
                groupDf,groupPeaks = processGroupedMouseImage(day+'-'+splitLuminescentFileName,luminescent,brightfield,brightfield2,sampleNames=sampleNames,visualize=visualize,save_images=save_images,save_pixel_df=save_pixel_df)
                if splitIndex != 0:
                    originalSamples = groupDf.index.unique('Sample').tolist()
                    newSamples = [str(int(x)+splitIndex) for x in originalSamples]
                    renamingDict = {}
                    for og,new in zip(originalSamples,newSamples):
                        renamingDict[og] = new
                    groupDf = groupDf.rename(renamingDict,level='Sample')
                
                splitIndex+=len(groupDf.index.unique('Sample').tolist())
                splitGroupDict[day+',,'+group+',,'+','.join(groupDf.index.unique('Sample').tolist())] = positionsToKeep
                groupDfList.append(groupDf)
                groupPeaksList+=groupPeaks
            groupDf = pd.concat(groupDfList)
            return groupDf,groupPeaksList,splitGroupDict           
        else:
            print('These images are not shared:\n')
            for missingImage in list(set(luminescentImages) ^ set(brightfieldImages)):
              print(day+'-'+missingImage)
            print('\nExiting...')
            sys.exit(0)
    else:
        #Read in luminescent image
        if group+'.png' in os.listdir(base_dir+'luminescent/'+day):
          fileName = base_dir+'luminescent/'+day+'/'+group+'.png'
        else:
          fileName = base_dir+'luminescent/'+day+'/'+group+'.PNG'
        luminescent = mplImage.imread(fileName)[:,:,:3]

        #Read in brightfield image
        if group+'.tif' in os.listdir(base_dir+'brightfield/'+day):
          fileName = base_dir+'brightfield/'+day+'/'+group+'.tif'
        else:
          fileName = base_dir+'brightfield/'+day+'/'+group+'.TIF'
        brightfield = mplImage.imread(fileName)
        brightfield2 = cv2.imread(fileName)        
        groupDf,groupPeaks = processGroupedMouseImage(day+'-'+group,luminescent,brightfield,brightfield2,sampleNames=sampleNames,visualize=visualize,save_images=save_images,save_pixel_df=save_pixel_df)
        return groupDf,groupPeaks,splitGroupDict
            
def fullInVivoImageProcessingPipeline(sampleNameFile,visualize=False,save_df=True,save_pixel_df=False,save_images=False):
  outputDir = 'outputData/'
  unmatchedGroups = luminescentBrightfieldMatchCheck(sampleNameFile,save_pixel_df=save_pixel_df)
  if len(unmatchedGroups) == 0:
    days = [x for x in pd.unique(sampleNameFile['Day'])]

    dayDfList,dayPixelDfList,dayPeaksList = [],[],[]
    fullSplitGroupDict = {}
    for i in trange(len(days), desc='Processing Days:'):
        day = days[i]
        groupDfList,groupPixelDfList = [],[]
        tempDf = sampleNameFile.query("Day == @day")
        groups = list(pd.unique([x for x in pd.unique(tempDf['Group'])]))
        #print(day)
        for j in trange(len(groups), desc='Processing Groups:',leave=False):
            group = groups[j]
            sampleNames = []
            
            groupDf,groupPeaks,splitGroupDict = checkSplitGroups(day,group,dayPeaksList,sampleNames=sampleNames,visualize=visualize,save_images=save_images,save_pixel_df=save_pixel_df)
            groupDfList.append(groupDf)
            dayPeaksList+=groupPeaks
            fullSplitGroupDict = {**fullSplitGroupDict,**splitGroupDict}
            #print(group)
        dayDf = pd.concat(groupDfList,keys=groups,names=['Group'])
        dayDfList.append(dayDf)

    experimentName = os.getcwd().split('/')[-1]
    outputFileName = 'radianceStatisticPickleFile-'+experimentName
    if 'SampleNames' not in tempDf.columns:
        sampleNamesColumn = []
    else:
        sampleNamesColumn = sampleNameFile['SampleNames'].tolist()
    
    fullDf = pd.concat(dayDfList,keys=days,names=['Day'])
    fullDf = amendSampleNames(fullDf,dayPeaksList,sampleNameFile,fullSplitGroupDict,save_pixel_df=save_pixel_df)
    radianceStatisticDf = addTrueIndexToDataframe(fullDf,sampleNameFile)
    radianceStatisticDf['Time'] = [int(x[1:]) for x in radianceStatisticDf.index.get_level_values('Day').tolist()]
    radianceStatisticDf = radianceStatisticDf.set_index(['Time'],append=True)
    if save_df:
      radianceStatisticDf.to_pickle(outputDir+outputFileName+'.pkl')
      radianceStatisticDf.to_excel(outputDir+outputFileName+'.xlsx')    
    return radianceStatisticDf
  else:
    print('These images are missing:\n')
    for missingImage in unmatchedGroups:
      print(missingImage)
    print('\nExiting...')
    return []

def selectMatrices(pMatrix,groups='all',days='all',samples='all'):
    if groups != 'all':
        groupValString = ','.join(groups)
    else:
        groupValString = 'all'
    groupTitle = '-'.join(['Group',groupValString])
    if days != 'all':
        dayValString = ','.join(days)
    else:
        dayValString = 'all'
    dayTitle = '-'.join(['Day',dayValString])
    selectionTitle = '_'.join([groupTitle,dayTitle])

    selectionDict = {}
    allDays,allGroups,allSamples = [x.split('-')[0] for x in pMatrix.files],[x.split('-')[1] for x in pMatrix.files],[x.split('-')[2] for x in pMatrix.files]
    if days == 'all':
        days = pd.unique(allDays).tolist()
    if groups == 'all':
        groups = pd.unique(allGroups).tolist()
    if samples == 'all':
        samples = pd.unique(allSamples).tolist()
    if type(days) != list:
        days = [days]
    if type(groups) != list:
        groups = [groups]
    if type(samples) != list:
        samples = [samples]
    selectionKeysList = []
    for day in days:
        for group in groups:
            for sample in samples:
                fullKey = '-'.join([day,group,sample])
                if fullKey in pMatrix.files:
                    selectionDict[fullKey] = pMatrix[fullKey]
                    selectionKeysList.append([day,group,sample])
                    
    selectionKeyMI = pd.MultiIndex.from_tuples(selectionKeysList,names=['Day','Group','Sample'])
    selectionKeyDf = pd.DataFrame(list(selectionDict.keys()),index=selectionKeyMI,columns=['Key'])    
    
    return selectionDict,selectionKeyDf,selectionTitle

def adaptMatricesForPlotting(radianceMatrix,brightfieldMatrix,trueMin):

    brightfieldAndBackgroundMatrix = np.add((radianceMatrix == -1).astype(int),(brightfieldMatrix == 0).astype(int))
    brightfieldAndBackgroundMatrix= (brightfieldAndBackgroundMatrix != 0).astype(int) 
    additionMatrix = np.multiply(brightfieldAndBackgroundMatrix,trueMin-1)
    plottingMatrix = np.multiply(radianceMatrix,1-brightfieldAndBackgroundMatrix) + additionMatrix            
    
    return plottingMatrix

def plotSingleMouseImage(axes,cmap,cbar_ax,pMatrixDict,minScaleDict,selectionKeysDf,row,col,r,c,rowVal,colVal,twoDaxes=True,groupRenamingDict={},marginTitles=True,numericDays=False,fontDict={}):
    trueVals,trueLevels,trueAxisIndices = [],[],[]
    for val,level,index in zip([rowVal,colVal],[row,col],[r,c]):
        if val != '':
            trueVals.append(val)
            trueLevels.append(level)
            trueAxisIndices.append(index)
    sampleKey = selectionKeysDf.xs(trueVals,level=trueLevels,drop_level=False).values[0,0]
    
    if len(groupRenamingDict) != 0:
        for i,level in enumerate(trueLevels):
            if level == 'Group' and trueVals[i] in groupRenamingDict.keys():
                trueVals[i] = groupRenamingDict[trueVals[i]]
    if numericDays:
        for i,level in enumerate(trueLevels):
            if level == 'Day':
                trueVals[i] = trueVals[i][1:]
    
    sampleMatrix = pMatrixDict[sampleKey]
    trueMin = minScaleDict[sampleKey][0]
    trueMax = minScaleDict[sampleKey][1]
    
    radianceMatrix,brightfieldMatrix,b = sampleMatrix[:,:,0],sampleMatrix[:,:,1],sampleMatrix[:,:,2]
    plottingMatrix = adaptMatricesForPlotting(radianceMatrix,brightfieldMatrix,trueMin)
    sampleDf = pd.DataFrame(plottingMatrix)
    sampleDf.iloc[-1,-1] = trueMax
    if twoDaxes:
        g = sns.heatmap(sampleDf,cmap=transparent_cmap(cmap),cbar = r == 0,ax=axes[r,c],norm=matplotlib.colors.LogNorm(),vmin=np.nanmin(sampleDf.values),vmax=trueMax,cbar_ax=cbar_ax,cbar_kws={'label':'Radiance (p/sec/cm$^2$/sr)'})
        if not marginTitles:
            axes[r,c].set_title('-'.join(trueVals),**fontDict)
        else:
            if r == 0:
                axes[r,c].set_title(trueVals[1],**fontDict)
            if c == 0:
                axes[r,c].text(-0.15,0.5,trueVals[0],verticalalignment='center',horizontalalignment='center',transform=axes[r,c].transAxes,**fontDict)
        axes[r,c].imshow(b,zorder=0, cmap='gray')
    else:
        g = sns.heatmap(sampleDf,cmap=transparent_cmap(cmap),cbar = trueAxisIndices[0] == 0,ax=axes[trueAxisIndices[0]],norm=matplotlib.colors.LogNorm(),vmin=np.nanmin(sampleDf.values),vmax=trueMax,cbar_ax=cbar_ax,cbar_kws={'label':'Radiance (p/sec/cm$^2$/sr)'})
        #TODO: Make faceting work with only one row or column as well
        if not marginTitles:
            axes[trueAxisIndices[0]].set_title(trueVals[1])
        else:
            axes[trueAxisIndices[0]].set_title(trueVals[1])
        axes[trueAxisIndices[0]].imshow(b,zorder=0, cmap='gray')

#Pad all 3 levels differently (-1, 0, minVal), then stack
def stackMouseImages(matrixList,orientation='h',unifiedPaddingShape = []):
    if len(unifiedPaddingShape) == 0:
        maxLength = max([x.shape[0] for x in matrixList])
        maxWidth = max([x.shape[1] for x in matrixList])
    else:
        maxLength = unifiedPaddingShape[0]
        maxWidth = unifiedPaddingShape[1]
    newMatrixList = []
    if orientation == 'h':
        for matrix in matrixList:
            dstackList = []
            for i,paddingConstant in enumerate([-1,0,np.min(matrix[:,:,2])]):
                newMatrix = np.multiply(np.ones([maxLength,matrix.shape[1]]),paddingConstant)
                newMatrix[:matrix.shape[0],:matrix.shape[1]] = matrix[:,:,i]
                dstackList.append(newMatrix)
            newMatrixList.append(np.dstack(dstackList))
        paddedMatrix = np.hstack(newMatrixList)
    else:
        for matrix in matrixList:
            dstackList = []
            for i,paddingConstant in enumerate([-1,0,np.min(matrix[:,:,2])]):
                newMatrix = np.multiply(np.ones([matrix.shape[0],maxWidth]),paddingConstant)
                newMatrix[:matrix.shape[0],:matrix.shape[1]] = matrix[:,:,i]
                dstackList.append(newMatrix)
            newMatrixList.append(np.dstack(dstackList))
        paddedMatrix = np.vstack(newMatrixList)
    if len(unifiedPaddingShape) != 0:
        dstackList = []
        for i,paddingConstant in enumerate([-1,0,np.min(paddedMatrix[:,:,2])]):
            newMatrix = np.multiply(np.ones([maxLength,maxWidth]),paddingConstant)
            newMatrix[:paddedMatrix.shape[0],:paddedMatrix.shape[1]] = paddedMatrix[:,:,i]
            dstackList.append(newMatrix)
        paddedMatrix = np.dstack(dstackList)
        
    return paddedMatrix

def concatenateImage(pMatrixDict,minScaleDict,selectionKeysDf,kwargDict,kwargValsDict,unifiedPaddingShape=[]):
    
    matrixList,minList,maxList = [],[],[]
    selectionKeysDf = pd.concat([selectionKeysDf],keys=['dummy'],names=['dummy'])
    #TODO: ADD DUMMY IMAGES (BLACK) TO INNERROW/INNERCOL PARAMETERS
    if kwargDict['innerRow'] != '':
        if kwargDict['innerCol'] != '':
            for val in kwargValsDict['innerRow']:
                if val in selectionKeysDf.index.unique(kwargDict['innerRow']):
                    matrixList2 = []
                    for val2 in kwargValsDict['innerCol']:
                        if val2 in selectionKeysDf.index.unique(kwargDict['innerCol']):
                            sampleKey = selectionKeysDf.xs([val,val2],level=[kwargDict['innerRow'],kwargDict['innerCol']],drop_level=False).values[0,0]
                            sampleMatrix,trueMin,trueMax = pMatrixDict[sampleKey],minScaleDict[sampleKey][0],minScaleDict[sampleKey][1]
                            matrixList2.append(sampleMatrix)
                            minList.append(trueMin)
                            maxList.append(trueMax)
                matrixList.append(stackMouseImages(matrixList2,orientation='h',unifiedPaddingShape = unifiedPaddingShape))
            fullMatrix = stackMouseImages(matrixList,orientation='v',unifiedPaddingShape = unifiedPaddingShape)
        else:
            for val in kwargValsDict['innerRow']:
                if val in selectionKeysDf.index.unique(kwargDict['innerRow']):
                    sampleKey = selectionKeysDf.xs([val],level=[kwargDict['innerRow']],drop_level=False).values[0,0]
                    sampleMatrix,trueMin,trueMax = pMatrixDict[sampleKey],minScaleDict[sampleKey][0],minScaleDict[sampleKey][1]
                else:
                    for tempIndex,tempVal in enumerate(kwargValsDict['innerRow']):
                        if tempVal in selectionKeysDf.index.unique(kwargDict['innerRow']):
                            break
                    sampleKey = selectionKeysDf.xs([selectionKeysDf.index.unique(kwargDict['innerRow']).tolist()[tempIndex]],level=[kwargDict['innerRow']],drop_level=False).values[0,0]
                    tempMatrix,trueMin,trueMax = pMatrixDict[sampleKey],minScaleDict[sampleKey][0],minScaleDict[sampleKey][1]
                    sampleMatrix = np.dstack([np.ones(tempMatrix.shape[:2])*-1,np.zeros(tempMatrix.shape[:2]),np.ones(tempMatrix.shape[:2])*np.min(tempMatrix[:,:,2])])

                matrixList.append(sampleMatrix)
                minList.append(trueMin)
                maxList.append(trueMax)
            fullMatrix = stackMouseImages(matrixList,orientation='v',unifiedPaddingShape = unifiedPaddingShape)
    else:
        if kwargDict['innerCol'] != '':
            for val2 in kwargValsDict['innerCol']:
                if val2 in selectionKeysDf.index.unique(kwargDict['innerCol']):
                    sampleKey = selectionKeysDf.xs([val2],level=[kwargDict['innerCol']],drop_level=False).values[0,0]
                    sampleMatrix,trueMin,trueMax = pMatrixDict[sampleKey],minScaleDict[sampleKey][0],minScaleDict[sampleKey][1]
                else:
                    for tempVal in kwargValsDict['innerCol']:
                        if tempVal in selectionKeysDf.index.unique(kwargDict['innerCol']):
                            tempIndex = selectionKeysDf.index.unique(kwargDict['innerCol']).tolist().index(tempVal)
                            break
                    sampleKey = selectionKeysDf.xs([selectionKeysDf.index.unique(kwargDict['innerCol']).tolist()[tempIndex]],level=[kwargDict['innerCol']],drop_level=False).values[0,0]
                    tempMatrix,trueMin,trueMax = pMatrixDict[sampleKey],minScaleDict[sampleKey][0],minScaleDict[sampleKey][1]
                    sampleMatrix = np.dstack([np.ones(tempMatrix.shape[:2])*-1,np.zeros(tempMatrix.shape[:2]),np.ones(tempMatrix.shape[:2])*np.min(tempMatrix[:,:,2])])
                matrixList.append(sampleMatrix) 
                minList.append(trueMin)
                maxList.append(trueMax)                
            fullMatrix = stackMouseImages(matrixList,orientation='h',unifiedPaddingShape = unifiedPaddingShape)
        else:
            fullMatrix = list(pMatrixDict.values())[0]
            minScale = list(minScaleDict.values())[0]
    
    return fullMatrix,[min(minList),max(maxList)]

def plotMouseImages(pMatrixDict,minScaleDict,selectionKeysDf,row='',col='',innerRow='',innerCol='',row_order=[],col_order=[],innerRowOrder=[],innerColOrder=[],cmap='magma',groupRenamingDict={},marginTitles=True,numericDays=True,useConstantImageSize=True,colorbarScale=2,font='Helvetica',fontsize=40,image_dir='',save_image=False,imageTitle=''):

    fontDict = {}
    for param,paramVal in zip(['fontname','fontsize'],[font,fontsize]):
        if paramVal != '':
            fontDict[param] = paramVal

    kwargDict = {'row':row,'col':col,'innerRow':innerRow,'innerCol':innerCol}
    kwargLenDict,kwargValsDict = {},{}
    for kwarg in kwargDict:
        if kwargDict[kwarg] != '' and len(selectionKeysDf.index.unique(kwargDict[kwarg])) != 1:
            kwargLenDict[kwarg] = len(selectionKeysDf.index.unique(kwargDict[kwarg]))
            #Make sure samples/days are sorted from lowest to highest
            if kwargDict[kwarg] == 'Sample':
                kwargValsDict[kwarg] = sorted(selectionKeysDf.index.unique(kwargDict[kwarg]).tolist())
            else:
                kwargValsDict[kwarg] = selectionKeysDf.index.unique(kwargDict[kwarg]).tolist()
        else:
            kwargLenDict[kwarg] = 1
            kwargValsDict[kwarg] = ['']
            kwargDict[kwarg] = ''

    if kwargLenDict['row'] == 1 or kwargLenDict['col'] == 1:
        twoDaxes = False
    else:
        twoDaxes = True

    if kwargDict['innerRow'] != '' or kwargDict['innerCol'] != '':
        wspace = 0.01
        hspace=0.01
        nonNestedVarLocations = [x for x in range(len(selectionKeysDf.index.names)) if list(selectionKeysDf.index.names)[x] in [kwargDict['row'],kwargDict['col']]]
        nestedVarLocations = [x for x in range(len(selectionKeysDf.index.names)) if list(selectionKeysDf.index.names)[x] in [kwargDict['innerRow'],kwargDict['innerCol']]]
        uniqueNestedVars,uniqueNonNestedVars = {},{}
        for rowIndex in range(selectionKeysDf.shape[0]):
            name = list(selectionKeysDf.iloc[rowIndex,:].name)
            nonNestedVars = '-'.join([name[x] for x in range(len(name)) if x in nonNestedVarLocations])
            nestedVars = '-'.join([name[x] for x in range(len(name)) if x in nestedVarLocations])
            if nonNestedVars not in list(uniqueNonNestedVars.keys()):
                uniqueNonNestedVars[nonNestedVars] = rowIndex
        newSelectionKeysDf = selectionKeysDf.iloc[list(uniqueNonNestedVars.values()),:].droplevel(nestedVarLocations)
        newSelectionKeysDf.iloc[:,0] = list(uniqueNonNestedVars.keys())

        newPmatrixDict,newMinScaleDict = {},{}
        for rowIndex in range(newSelectionKeysDf.shape[0]):
            levelValues = list(newSelectionKeysDf.iloc[rowIndex,:].name)
            levels = list(newSelectionKeysDf.index.names)
            keysToCombine = selectionKeysDf.xs(levelValues,level=levels)
            newKey = newSelectionKeysDf.iloc[rowIndex,0]
            concatenatedImage,minScale = concatenateImage(pMatrixDict,minScaleDict,keysToCombine,kwargDict,kwargValsDict)
            newPmatrixDict[newKey] = concatenatedImage
            newMinScaleDict[newKey] = minScale
        if useConstantImageSize:
            unifiedPadLength = max([newPmatrixDict[x].shape[0] for x in newPmatrixDict])
            unifiedPadWidth = max([newPmatrixDict[x].shape[1] for x in newPmatrixDict])
            unifiedPaddingShape = [unifiedPadLength,unifiedPadWidth]
            for rowIndex in range(newSelectionKeysDf.shape[0]):
                levelValues = list(newSelectionKeysDf.iloc[rowIndex,:].name)
                levels = list(newSelectionKeysDf.index.names)
                keysToCombine = selectionKeysDf.xs(levelValues,level=levels)
                newKey = newSelectionKeysDf.iloc[rowIndex,0]
                concatenatedImage,minScale = concatenateImage(pMatrixDict,minScaleDict,keysToCombine,kwargDict,kwargValsDict,unifiedPaddingShape=unifiedPaddingShape)
                newPmatrixDict[newKey] = concatenatedImage

        selectionKeysDf = newSelectionKeysDf.copy()
        pMatrixDict = newPmatrixDict.copy()
        minScaleDict = newMinScaleDict.copy()
    else:
        wspace = None
        hspace=None

    plottedParameterIndices = [list(selectionKeysDf.index.names).index(x) for x in selectionKeysDf.index.names if x in [kwargDict['row'],kwargDict['col']]]
    plottedParameterTuples = []
    for indexTuple in selectionKeysDf.values[:,0].tolist():
        tempList = []
        indexTupleList = indexTuple.split('-')
        for index in plottedParameterIndices:
            tempList.append(indexTupleList[index])
        plottedParameterTuples.append(set(tempList))

    fig, axes = plt.subplots(kwargLenDict['row'],kwargLenDict['col'],figsize=(2.5*kwargLenDict['col']*kwargLenDict['innerCol']*0.5,4.55*kwargLenDict['row']*kwargLenDict['innerRow']))
    fig.subplots_adjust(right=0.8)

    if not marginTitles:
        titleList = [x for x in [kwargDict['row'],kwargDict['col']] if x != '']
        fig.suptitle('-'.join(titleList),**fontDict)
        fig.subplots_adjust(top=1-0.25/kwargLenDict['row'],wspace=wspace)
    else:
        fig.subplots_adjust(wspace=wspace,hspace=hspace)

    fontDict2 = fontDict.copy()
    fontDict2['fontweight'] = 'bold'
    if 'fontsize' in fontDict2:
        fontDict2['fontsize'] = fontDict2['fontsize']*1.2
    else:
        fontDict2['fontsize'] = 12
    levelTitles = []
    if kwargDict['row'] != '':
        if kwargLenDict['col'] != 1:
            axbox1 = axes[0,0].get_position().extents
        else:
            axbox1 = axes[0].get_position().extents
        rightXPos = axbox1[0]
        a1 = plt.figtext(rightXPos,0.5,kwargDict['row']+'\n\n', rotation = 90,horizontalalignment='right',verticalalignment='center',**fontDict2)
        levelTitles.append(a1)
        #fig.suptitle(kwargDict['row']+'\n',x=rightXPos,y=0.5, rotation = 90,horizontalalignment='right',verticalalignment='center',**fontDict2)
    if kwargDict['col'] != '':
        if kwargLenDict['col'] % 2 == 0:
            if kwargLenDict['row'] != 1:
                axbox1 = axes[0,int(kwargLenDict['col']/2) -1].get_position().extents
                axbox2 = axes[0,int(kwargLenDict['col']/2)].get_position().extents
            else:
                axbox1 = axes[int(kwargLenDict['col']/2) -1].get_position().extents
                axbox2 = axes[int(kwargLenDict['col']/2)].get_position().extents
            middleXPos = 0.5*(0.5*(axbox1[0] + axbox1[2]) + 0.5*(axbox2[0] + axbox2[2]))
        else:
            if kwargLenDict['row'] != 1:
                axbox1 = axes[0,int(kwargLenDict['col']/2)].get_position().extents
            else:
                axbox1 = axes[int(kwargLenDict['col']/2)].get_position().extents
            middleXPos = 0.5*(axbox1[0] + axbox1[2])
        bottomYPos = axbox1[3]
        a2 = plt.figtext(middleXPos,bottomYPos,kwargDict['col']+'\n',horizontalalignment='center',verticalalignment='bottom',**fontDict2)
        levelTitles.append(a2)

    barWidth = colorbarScale*0.02*(2/len(kwargValsDict['col']))
    barHeight = colorbarScale*0.8*(1/len(kwargValsDict['row']))
    cbar_ax = fig.add_axes([0.86-0.005*len(kwargValsDict['col']), 0.5-(0.1+barHeight/2)+0.1, barWidth, barHeight])

    #cbar_ax = fig.add_axes([0.85, 0.15, 0.02, 0.7])
    cmap = sns.color_palette(cmap, as_cmap=True)
    for r,rowVal in enumerate(kwargValsDict['row']):
        for c,colVal in enumerate(kwargValsDict['col']):
            if not twoDaxes:
                if kwargLenDict['row'] == 1:
                    plottedParameterList = [colVal]
                    axes[c].axis('off')
                else:
                    plottedParameterList = [rowVal]
                    axes[r].axis('off')
            else:
                plottedParameterList = [rowVal,colVal]
                axes[r,c].axis('off')

            if set(plottedParameterList) in plottedParameterTuples:
                plotSingleMouseImage(axes,cmap,cbar_ax,pMatrixDict,minScaleDict,selectionKeysDf,kwargDict['row'],kwargDict['col'],r,c,rowVal,colVal,twoDaxes=twoDaxes,groupRenamingDict=groupRenamingDict,marginTitles=marginTitles,numericDays=numericDays,fontDict=fontDict)
            else:
                if marginTitles:
                    trueVals,trueLevels,trueAxisIndices = [],[],[]
                    for val,level,index in zip([rowVal,colVal],[row,col],[r,c]):
                        if val != '':
                            trueVals.append(val)
                            trueLevels.append(level)
                            trueAxisIndices.append(index)
                    if len(groupRenamingDict) != 0:
                        for i,level in enumerate(trueLevels):
                            if level == 'Group' and trueVals[i] in groupRenamingDict.keys():
                                trueVals[i] = groupRenamingDict[trueVals[i]]
                    if numericDays:
                        for i,level in enumerate(trueLevels):
                            if level == 'Day':
                                trueVals[i] = trueVals[i][1:]
                    if r == 0:
                        axes[r,c].set_title(trueVals[1],**fontDict)
                    if c == 0:
                        axes[r,c].text(0,0.5,trueVals[0],verticalalignment='center',horizontalalignment='right',transform=axes[r,c].transAxes,**fontDict)
    cbar_ax.set_frame_on(True)
    cbar_ax.tick_params(which='both',width=colorbarScale*1.5)
    if fontsize != '':
        cbar_ax.yaxis.label.set_fontsize(fontsize)
        for l in cbar_ax.yaxis.get_ticklabels():
            l.set_fontsize(fontsize)
    if font != '':
        cbar_ax.yaxis.label.set_family(font)
        for l in cbar_ax.yaxis.get_ticklabels():
            l.set_family(font)
    if save_image:
        paramTitleList = []
        for param in kwargDict:
            if kwargDict[param] != '':
                paramTitleList.append('-'.join([param,kwargDict[param]]))
        paramTitle = '_'.join(paramTitleList)
        experimentName = os.getcwd().split('/')[-1]
        imageTitle = '_'.join(['mouseImage',experimentName,imageTitle,paramTitle])
        fig.savefig('plots/'+imageTitle+'.png',bbox_extra_artists=(cbar_ax,*levelTitles),bbox_inches='tight')



#!/usr/bin/env python3
import os,itertools,subprocess
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
from operator import itemgetter
import plateypus.plotting.facetPlot1D as fp1D
import plateypus.plotting.facetPlotCategorical as fpCategorical
import plateypus.plotting.facetPlot2D as fp2D
import plateypus.plotting.facetPlot3D as fp3D
from plateypus.dataprocessing.miscFunctions import sortSINumerically

idx = pd.IndexSlice
plotFolderName = 'plots'
def produceSubsettedDataFrames(fulldf,withinFigureBoolean,specificValueBooleanList,trueLabelDict):
    fulldf = fulldf.astype(float)
    
    #Get all possible subsetted indices
    figureSubsettedLevelValues = []
    withinFigureSubsettedLevelValues = []
    figureSubsettingLevels = []
    figureLevelNames = []
    figureLevelIndices = []
    levelValuesPlottedIndividually = []
    for levelIndex,currentLevelName in enumerate(fulldf.index.names):
        #Event level always has every value included (single cell)
        #Otherwise, check which level values in the level were selected by the user
        if currentLevelName != 'Event':
            #currentLevelValues = pd.unique(fulldf.index.get_level_values(currentLevelName))
            currentLevelValues = trueLabelDict[currentLevelName]
            levelValues = []
            if (currentLevelName == 'Marker' or currentLevelName == 'Markers') and 'Event' in fulldf.index.names:
                realLevelIndex = levelIndex - 1
            else:
                realLevelIndex = levelIndex
            #Go through each level value in the level; regardless of figure selection status, and add based on level value selection status
            for levelValue,specificBoolean in zip(currentLevelValues,specificValueBooleanList[realLevelIndex]):
                if specificBoolean:
                    levelValues.append(levelValue)
            #If we will include this level within the figure
            if withinFigureBoolean[realLevelIndex]:
                withinFigureSubsettedLevelValues.append(levelValues)
                if len(levelValues) == len(currentLevelValues):
                    for levelValue in levelValues:
                        levelValuesPlottedIndividually.append(levelValue)
            #Only need to add level values to figure list; will be xs'd out of the full dataframe in the subsetted, within figure dataframes
            else:
                figureSubsettedLevelValues.append(levelValues)
                figureLevelNames.append(currentLevelName)
                figureLevelIndices.append(levelIndex)
    if len(figureLevelIndices) > 0:
        #Get all row level values present in the dataframe
        allPossibleSubsettingCombos = itertools.product(*figureSubsettedLevelValues)
        if 'Event' not in fulldf.index.names:
            rowList = []
            for row in range(fulldf.shape[0]):
                allCurrentLevelValues = fulldf.iloc[row,:].name
                currentLevelValues = itemgetter(*figureLevelIndices)(allCurrentLevelValues)
                if not isinstance(currentLevelValues,tuple):
                    currentLevelValues = tuple([currentLevelValues])
                rowList.append(currentLevelValues)
            actualSubsettingCombos = []
            #From original dataframe; select all rows that appear in the all possible combination list 
            for subsettingCombo in allPossibleSubsettingCombos:
                if subsettingCombo in rowList:
                    actualSubsettingCombos.append(subsettingCombo) 
        else:
            rowList = []
            nonEventLevelList = []
            for level in fulldf.index.names:
                if level != 'Event':
                    nonEventLevelList.append(level)
            noEventDf = fulldf.groupby(nonEventLevelList).first()
            for row in range(noEventDf.shape[0]):
                allCurrentLevelValues = noEventDf.iloc[row,:].name
                figureLevelIndices = [max(min(x, len(fulldf.index.names)-2), 0) for x in figureLevelIndices]
                currentLevelValues = itemgetter(*figureLevelIndices)(allCurrentLevelValues)
                if not isinstance(currentLevelValues,tuple):
                    currentLevelValues = tuple([currentLevelValues])
                rowList.append(currentLevelValues)
            actualSubsettingCombos = []
            #From original dataframe; select all rows that appear in the all possible combination list 
            for subsettingCombo in allPossibleSubsettingCombos:
                if subsettingCombo in rowList:
                    actualSubsettingCombos.append(subsettingCombo) 
            #actualSubsettingCombos = allPossibleSubsettingCombos
        #Use these levels to cross section the fulldf, generating a list of subsetted dfs that will each have their own figure
        allPossibleSubsettedDfList = []
        for actualSubsettingCombo in actualSubsettingCombos:
            possibleSubsettedDf = fulldf.xs(actualSubsettingCombo, level=figureLevelNames)
            allPossibleSubsettedDfList.append(possibleSubsettedDf)
    else:
        actualSubsettingCombos = ['All']
        allPossibleSubsettedDfList = [fulldf]
    actualLevelValueDfList = []
    #Go through each subsetteddf, and only grab rows with level values that are selected
    for possibleSubsettedDf in allPossibleSubsettedDfList:
        allPossibleLevelValueCombos = itertools.product(*withinFigureSubsettedLevelValues)
        if 'Event' not in fulldf.index.names:
            rowList = []
            for row in range(possibleSubsettedDf.shape[0]):
                allCurrentLevelValues = possibleSubsettedDf.iloc[row,:].name
                rowList.append(allCurrentLevelValues)
            actualLevelValueCombos = []
            #From original dataframe; select all rows that appear in the all possible level value combination list 
            levelValueRowList = []
            for levelValueCombo in allPossibleLevelValueCombos:
                if levelValueCombo in rowList:
                    indices = [i for i, x in enumerate(rowList) if x == levelValueCombo]
                    levelValueRowList+=indices
            actualLevelValueDf = possibleSubsettedDf.iloc[levelValueRowList,:]
            actualLevelValueDfList.append(actualLevelValueDf)
        else:
            unstackedDf = possibleSubsettedDf.unstack('Event')
            allLevelValues = []
            for row in range(unstackedDf.shape[0]):
                allCurrentLevelValues = unstackedDf.iloc[row,:].name
                allLevelValues.append(allCurrentLevelValues)
            realLevelValueCombos = []
            for possibleLevelValueCombo in allPossibleLevelValueCombos:
                if possibleLevelValueCombo in allLevelValues:
                    realLevelValueCombos.append(possibleLevelValueCombo)
            subsettingList = list(map(list, zip(*realLevelValueCombos)))
            realSubsettingList = []
            for subset in subsettingList:
                newSubset = list(pd.unique(subset))
                realSubsettingList.append(newSubset)
            actualSubsettingList = []
            realsubset = 0
            for subset,levelName in enumerate(possibleSubsettedDf.index.names):
                if levelName == 'Event':
                    actualSubsettingList.append(slice(None))
                else:
                    actualSubsettingList.append(realSubsettingList[realsubset])
                    realsubset+=1
            actualLevelValueDf = possibleSubsettedDf.loc[tuple(actualSubsettingList),:]
            actualLevelValueDfList.append(actualLevelValueDf)
    if not isinstance(actualLevelValueDfList,list):
        actualLevelValueDfList = [actualLevelValueDfList]
    if not isinstance(actualSubsettingCombos,list):
        actualSubsettingCombos = ['All']

    #Remove columns from non postprocessed data
    if 'Dimension' not in fulldf.columns[0]:
        for i in range(len(actualLevelValueDfList)):
            actualLevelValueDfList[i].columns.name = ''
    return actualLevelValueDfList,actualSubsettingCombos,figureLevelNames,levelValuesPlottedIndividually

def createFacetPlotName(folderName,dataType,plotType,subPlotType,legendParameterToLevelNameDict,subsettedDfTitle,levelsPlottedIndividually,useModifiedDf,plotOptions):
    delimiter1 = '-'
    delimiter2 = ','
    
    legendParameterString = delimiter2.join(list(legendParameterToLevelNameDict.keys()))

    flattened_list = []
    for val in legendParameterToLevelNameDict.values():
        if isinstance(val, (list,)):
            for val2 in val:
                flattened_list.append(val2)
        else:
            flattened_list.append(val)

    levelNameString = delimiter2.join(flattened_list)
    figureLevelNameString = delimiter2.join(list(map(str,subsettedDfTitle)))

    if len(levelsPlottedIndividually) == 0:
        individualLevelString = 'all'
    else:
        individualLevelString = delimiter2.join(list(map(str,levelsPlottedIndividually)))
    if useModifiedDf:
        modifiedString = '-modified'
    else:
        modifiedString = ''

    axisScalingStringList = []
    for axis in plotOptions:
        if 'X' in axis:
            if plotOptions[axis]['axisScaling'] == 'Logarithmic':
                axisScalingStringList.append('logX')
            elif plotOptions[axis]['axisScaling'] == 'Biexponential':
                axisScalingStringList.append('biexpX')
            else:
                axisScalingStringList.append('linX')
        else:
            if plotOptions[axis]['axisScaling'] == 'Logarithmic':
                axisScalingStringList.append('logY')
            elif plotOptions[axis]['axisScaling'] == 'Biexponential':
                axisScalingStringList.append('biexpY')
            else:
                axisScalingStringList.append('linY')
    axisScalingString = delimiter2.join(axisScalingStringList)

    if len(subsettedDfTitle) == 0 or subsettedDfTitle == 'All':
        initialString = delimiter1.join([subPlotType,dataType,folderName,legendParameterString,levelNameString,axisScalingString])
    else:
        initialString = delimiter1.join([subPlotType,dataType,folderName,legendParameterString,levelNameString,figureLevelNameString,axisScalingString])
    fullTitleString = initialString+modifiedString
    if '/' in fullTitleString:
        fullTitleString = fullTitleString.replace("/", "_")
    if '.' in fullTitleString:
        fullTitleString = fullTitleString.replace(".", "_")
    if ' ' in fullTitleString:
        fullTitleString = fullTitleString.replace(" ", "_")
    return fullTitleString

def subsetOriginalLevelValueOrders(unsubsettedOrders,subsettedDf,parameter):
    subsettedValues = list(pd.unique(subsettedDf.index.get_level_values(parameter)))
    subsettedOrders = []
    for unsubsettedValue in unsubsettedOrders:
        if unsubsettedValue in subsettedValues:
            subsettedOrders.append(unsubsettedValue)
    return subsettedOrders

def plotFacetedFigures(folderName,plotType,subPlotType,dataType,subsettedDfList,subsettedDfListTitles,figureParameters,levelsPlottedIndividually,useModifiedDf,fulldf,plotOptions,legendParameterToLevelNameDict,addDistributionPoints,alternateTitle='',originalLevelValueOrders = {},subfolderName='',context='notebook',height=3,aspect=1,titleBool='yes',colwrap=5,legendBool='yes',cmap='',outlierZScore=3,plotAllVar=True,titleAdjust='',plotSpecificDict={}):
    sns.set_context(context)
    sns.set_palette(sns.color_palette())
    global hei,asp,titleVar,legendVar
    hei = height
    asp = aspect
    titleVar = titleBool
    legendVar = legendBool
    zScoreCutoff = 3
    #Has issues with repeated values (aka CD54 shows up in TCells and APCs)
    for subsettedDf,subsettedDfTitle in zip(subsettedDfList,subsettedDfListTitles):
        #if outlierBool == 'yes':
        #    beforeOutlierRemoval = subsettedDf.shape[0]
        #    if plotOptions['Y']['axisScaling'] == 'Logarithmic':
        #        minVal = min(subsettedDf.values)
        #        if minVal <= 0:
        #            newSubsettedDf = subsettedDf.iloc[:,:]+abs(minVal)+1
        #        else:
        #            newSubsettedDf = subsettedDf.copy()
        #        newSubsettedDf = np.log10(newSubsettedDf)
        #        subsettedDf = subsettedDf[(np.abs(stats.zscore(newSubsettedDf)) < outlierZScore).all(axis=1)]
        #    else:
        #        subsettedDf = subsettedDf[(np.abs(stats.zscore(subsettedDf)) < outlierZScore).all(axis=1)]
        #    afterOutlierRemoval = subsettedDf.shape[0]
        #    print(str(beforeOutlierRemoval-afterOutlierRemoval)+' outliers removed!')
            
        #Assign all levels to plot parameters in catplot/relplot; reassign x/y axis level names to desired x/y axis titles
        kwargs = {}
        facetgridkwargs = {}
        for parameter in legendParameterToLevelNameDict:
            if parameter == 'Y Axis Values':
                if subPlotType == 'heatmap':
                    kwargs['y'] = legendParameterToLevelNameDict[parameter]
            else:
                currentLevel = legendParameterToLevelNameDict[parameter]
            if parameter == 'Color':
                kwargs['hue'] = currentLevel
                if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                    kwargs['hue_order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['hue']],subsettedDf,kwargs['hue'])
            elif parameter == 'Size':
                kwargs['size'] = currentLevel
                if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                    kwargs['size_order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['size']],subsettedDf,kwargs['size'])
            elif parameter == 'Marker':
                kwargs['style'] = currentLevel
            elif parameter == 'Order':
                subsettedDf.index.set_names(plotOptions['X']['axisTitle'],level=currentLevel,inplace=True)
                kwargs['x'] = plotOptions['X']['axisTitle']
                if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                    kwargs['order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['x']],subsettedDf,kwargs['x'])
            elif parameter == 'Column':
                kwargs['col'] = currentLevel
                unorderedLevelValues = list(pd.unique(subsettedDf.index.get_level_values(currentLevel)))
                if 'Concentration' in currentLevel and 'M' in unorderedLevelValues[0]:
                    levelValues = sortSINumerically(unorderedLevelValues,True,True)[0]
                    kwargs['col_order'] = levelValues 
                else:
                    if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                        kwargs['col_order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['col']],subsettedDf,kwargs['col'])
                    else:
                        kwargs['col_order'] = unorderedLevelValues
                facetgridkwargs['col'] = kwargs['col']
                facetgridkwargs['col_order'] = kwargs['col_order']
            elif parameter == 'Row':
                kwargs['row'] = currentLevel
                unorderedLevelValues = list(pd.unique(subsettedDf.index.get_level_values(currentLevel)))
                if 'Concentration' in currentLevel and 'M' in unorderedLevelValues[0]:
                    levelValues = sortSINumerically(unorderedLevelValues,True,True)[0]
                    kwargs['row_order'] = levelValues 
                else:
                    levelValues = unorderedLevelValues
                    if len(originalLevelValueOrders) != 0 and currentLevel in originalLevelValueOrders.keys():
                        kwargs['row_order'] = subsetOriginalLevelValueOrders(originalLevelValueOrders[kwargs['row']],subsettedDf,kwargs['row'])
                    else:
                        kwargs['row_order'] = unorderedLevelValues 
                facetgridkwargs['row'] = kwargs['row']
                facetgridkwargs['row_order'] = kwargs['row_order']
            elif parameter == 'X Axis Values':
                if len(plotOptions['X']['axisTitle']) > 1 and dataType != 'dr':
                    subsettedDf.index.set_names(plotOptions['X']['axisTitle'],level=currentLevel,inplace=True)
                    kwargs['x'] = plotOptions['X']['axisTitle']
                else:
                    kwargs['x'] = currentLevel
            else:
                pass
        #Assign y axis (or color bar axis) parameter
        #if subPlotType not in ['kde','histogram']:
        if dataType == 'dr':
            kwargs['y'] = plotOptions['Y']['axisTitle']
        else:
            if subPlotType != 'heatmap':
                if 'Statistic' in figureParameters:
                    subsettedDf.columns = [subsettedDfTitle[figureParameters.index('Statistic')]]
                    kwargs['y'] = subsettedDfTitle[figureParameters.index('Statistic')]
                else:
                    subsettedDf.columns = [plotOptions['Y']['axisTitle']]
                    kwargs['y'] = plotOptions['Y']['axisTitle']
            else:
                if 'Statistic' in figureParameters:
                    subsettedDf.columns = [subsettedDfTitle[figureParameters.index('Statistic')]]
                    kwargs['z'] = subsettedDfTitle[figureParameters.index('Statistic')]
                else:
                    subsettedDf.columns = [plotOptions['Colorbar']['axisTitle']]
                    kwargs['z'] = plotOptions['Colorbar']['axisTitle']
        
        if dataType == 'singlecell':
            plottingDf = subsettedDf.copy()
            #plottingDf = subsettedDf.stack().to_frame('GFI')
        else:
            plottingDf = subsettedDf.copy()
        #Converts wide form dataframe into long form required for cat/relplot
        plottingDf = plottingDf.reset_index()
        #Use plot options file to initialize numeric x axis ordering
        #NEED TO GET WORKING WITH HEATMAPS
        if subPlotType != 'heatmap':
            if plotType != '1d':
                if plotOptions['X']['numeric']:
                    currentLevelValues = list(plottingDf[kwargs['x']])
                    sortedOldLevelValues,newLevelValues = sortSINumerically(currentLevelValues,False,True)
                    #Need to interpret parenthetical units for x to get 1e9
                    if 'M)' in kwargs['x']:
                        s = kwargs['x']
                        units = '1'+s[s.find("(")+1:s.find(")")]
                        scaledSortedUnits,sortedUnits = sortSINumerically([units],False,True)
                    else:
                        sortedUnits = [1]
                    scaledNewLevelValues = [float(i) / float(sortedUnits[0]) for i in newLevelValues]
                    plottingDf[kwargs['x']] = scaledNewLevelValues
                    if plotType == 'categorical':
                        kwargs['order'] = scaledNewLevelValues
        else:
            pass
        col_wrap_min = colwrap
        #Make sure there are not more than 6 plots in a row (if no row facet variable)
        if 'row' not in facetgridkwargs.keys():
            if 'col' in facetgridkwargs.keys():
                colwrap = min([len(kwargs['col_order']),col_wrap_min])
                kwargs['col_wrap'] = colwrap
        
        if plotAllVar:
            fullTitleString = createFacetPlotName(folderName,dataType,plotType,subPlotType,legendParameterToLevelNameDict,subsettedDfTitle,levelsPlottedIndividually,useModifiedDf,plotOptions)
            if 'temporaryFirstPlot.png' in os.listdir('plots'):
                subprocess.run(['rm','plots/temporaryFirstPlot.png'])
        else:
            fullTitleString = 'temporaryFirstPlot'
        if len(subsettedDf.index) > 0:
            plotSubsettedFigure(subsettedDf,plottingDf,kwargs,facetgridkwargs,plotSpecificDict,plotType,subPlotType,dataType,fullTitleString,plotOptions,subsettedDfTitle,addDistributionPoints,alternateTitle,subfolderName,titleAdjust,cmap)
        if not plotAllVar:
            break
    sns.set_context('notebook')

def reorderKwargs(oldKwargs,kwargs):
    reorderedKwargs = {}
    for key in kwargs:
        if key in oldKwargs:
            reorderedList = []
            if isinstance(oldKwargs[key],list) or isinstance(oldKwargs[key],tuple):
                for val in oldKwargs[key]:
                    for val2 in kwargs[key]:
                        if isinstance(val,str):
                            if val == val2.strip():
                                reorderedList.append(val2)
                                break
                        else:
                            print('val not in val2')
                if len(reorderedList) == 0:
                    reorderedKwargs[key] = kwargs[key]
                else:
                    reorderedKwargs[key] = reorderedList
            else:
                reorderedKwargs[key] = kwargs[key]
        else:
            reorderedKwargs[key] = kwargs[key]
    return reorderedKwargs

#Will not be needed when seaborn 0.9.1 releases
def sanitizeSameValueLevels(plottingDf,kwargs):
    oldKwargs = kwargs.copy()
    unsanitizedPlottingDf = plottingDf.copy()
    #First find levels that have values that are the same as other levels:
    #Get numeric axes levels
    numericAxes = []
    for axis in ['x','y','z']:
        if axis in kwargs.keys():
            numericAxes.append(kwargs[axis])
    #Exclude levels used for numeric axes (x,y,color)
    nonNumericNameList = []
    for levelName in plottingDf.columns:
        if levelName not in numericAxes:
            nonNumericNameList.append(levelName)
    #Get all possible pairs of nonnumericlevels
    possiblePairs = list(itertools.combinations(nonNumericNameList,2))
    overlappingPairs = []
    for possiblePair in possiblePairs:
        uniqueLevelValues1 = pd.unique(plottingDf[possiblePair[0]])
        uniqueLevelValues2 = pd.unique(plottingDf[possiblePair[1]])
        #If overlap, add both levelnames to list
        if bool(set(uniqueLevelValues1) & set(uniqueLevelValues2)):
            overlappingPairs.append(possiblePair)
    #Add dummy spaces to make level values different while not affecting the legend
    #NEED TO THINK ABOUT HOW THIS AFFECTS NUMERIC VARIABLES
    #NEED TO THINK ABOUT HOW TO DEAL WITH VARIABLES I NEED TO SORT NUMERICALLY; MOST LIKELY BY DETERMINING ORDER BEFORE SANITIZING AND ADDING SPACES TO ORDER
    for overlappingPair in overlappingPairs:
        newVals = []
        for value in plottingDf[overlappingPair[1]]:
            newval = str(value) + ' '
            newVals.append(newval)
        plottingDf[overlappingPair[1]] = newVals
        for kwarg,kwargParent in zip(['row_order','col_order','hue_order','size_order'],['row','col','hue','size']):
            if kwarg in kwargs.keys():
                if kwargs[kwargParent] == overlappingPair[1]:
                    kwargs[kwarg] = list(pd.unique(plottingDf[overlappingPair[1]]))
    kwargs = reorderKwargs(oldKwargs,kwargs)
    return plottingDf,kwargs

def plotSubsettedFigure(subsettedDf,plottingDf,kwargs,facetgridkwargs,plotSpecificKwargs,plotType,subPlotType,dataType,fullTitleString,plotOptions,subsettedDfTitle,addDistributionPoints,alternateTitle,subfolderName,titleAdjust,cmap):

    titleBool = True
    secondPathBool = False
    
    #Add in sharex/sharey options
    if plotType != '1d':
        facetKwargs = {'sharex':plotOptions['X']['share'],'sharey':plotOptions['Y']['share']}
    else:
        facetKwargs = {'sharex':False,'sharey':plotOptions['Y']['share']}
    
    if alternateTitle !=  '':
        fullTitleString = alternateTitle 

    auxillaryKwargs = {}
    auxillaryKwargs['plotType'] = plotType
    auxillaryKwargs['subPlotType'] = subPlotType
    auxillaryKwargs['facetgridkwargs'] = facetgridkwargs
    auxillaryKwargs['plotspecifickwargs'] = plotSpecificKwargs
    if cmap != '':
        if cmap == 'glasbey':
            import colorcet as cc
            cmap = cc.glasbey[:len(pd.unique(plottingDf[kwargs['hue']]))]
        cmapKwarg = {'palette':cmap}
    else:
        cmapKwarg = {}
    auxillaryKwargs['cmap'] = cmapKwarg
    
    #if hei != 5 and asp != 1:
    if plotType == '1d':
        plotOptions['Y']['figureDimensions'] = {'height':hei,'aspect':asp}
    else:
        plotOptions['X']['figureDimensions'] = {'height':hei,'aspect':asp}
    #else:
    #    plotOptions['X']['figureDimensions'] = {}
    #Use appropriate facet plot
    #1D plots: (Histograms and KDEs); use facetgrid with appropriate axis level seaborn function
    if plotType == '1d':
        auxillaryKwargs['dataType'] = dataType
        facetPlotType = fp1D
        #Fix duck typing issue with replots: https://github.com/mwaskom/seaborn/issues/1653
        if 'hue' in kwargs and isinstance(plottingDf[kwargs['hue']][0],str):
            if plottingDf[kwargs['hue']][0].isnumeric():
                for i,x in enumerate(kwargs['hue_order']):
                    kwargs['hue_order'][i] = "$%s$" % x
                plottingDf[kwargs['hue']] = ["$%s$" % x for x in plottingDf[kwargs['hue']]]
    #1.5D/categorical plots (bar/point/box etc.); use catplot figure level seaborn function
    elif plotType == 'categorical':
        auxillaryKwargs['addDistributionPoints'] = addDistributionPoints
        facetPlotType = fpCategorical
    #2D plots (line and scatter); use relplot figure level seaborn function
    elif plotType == '2d':
        facetPlotType = fp2D
    #3D plots (heatmaps and 3d scatter/lineplots); use facet grid with appropriate axis level seaborn function
    elif plotType == '3d':
        facetPlotType = fp3D
    fg = facetPlotType.plot(plottingDf,subsettedDf,kwargs,facetKwargs,auxillaryKwargs,plotOptions)
    #SupTitle
    if titleVar == 'yes':
        #Make room for suptitle
        if titleAdjust == '':
            if 'row' in kwargs:
                if len(kwargs['row_order']) <=2:
                    plt.subplots_adjust(top=0.9)
                else:
                    plt.subplots_adjust(top=0.9+len(kwargs['row_order'])*0.005)
            else:
                plt.subplots_adjust(top=0.8)
        else:
            plt.subplots_adjust(top=float(titleAdjust))
        if subsettedDfTitle != 'All':
            subsettedDfTitle = list(map(str,subsettedDfTitle))
        else:
            subsettedDfTitle = [subsettedDfTitle]
        #Do not include placeholder celltypes in suptitle
        if 'NotApplicable' in subsettedDfTitle:
            subsettedDfTitle.remove('NotApplicable')
        plt.suptitle('-'.join(subsettedDfTitle),fontsize = 'x-large',fontweight='bold')
    
    #Legend
    if legendVar == 'no':
        fg._legend.remove()

    #Save figure
    if subfolderName != '':
        if subfolderName not in os.listdir(plotFolderName):
            subprocess.run(['mkdir',plotFolderName+'/'+subfolderName])
        fg.fig.savefig(plotFolderName+'/'+subfolderName+'/'+fullTitleString+'.png',bbox_inches='tight')
    else:
        fg.fig.savefig(plotFolderName+'/'+fullTitleString+'.png',bbox_inches='tight')
    if secondPathBool:
        fg.fig.savefig('../../outputFigures/'+fullTitleString+'.png',bbox_inches='tight')
    print(fullTitleString+' plot saved')
    plt.clf()



#!/usr/bin/env python3
import os,subprocess
import tkinter as tk

class RemoveExperimentWindow(tk.Frame):
    def __init__(self,master,backpage):
        tk.Frame.__init__(self, master)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10)
        
        l1 = tk.Label(mainWindow, text="Experiment Name: ")
        experiments = []
        for experimentName in os.listdir('experiments'):
            if '.DS' not in experimentName:
                experiments.append(experimentName)

        selectionVar = tk.StringVar()
        selectionMenu = tk.OptionMenu(mainWindow,selectionVar,*experiments)
        selectionMenu.grid(row=0,column=1)

        def removeExperiment():
            experimentName = selectionMenu.get()
            subprocess.run(['rm','-r','experiments/'+experimentName])

        b = tk.Button(mainWindow,text='Remove experiment',command=lambda:removeExperiment())
        
        l1.grid(row=0,column=0,sticky=tk.W)
        b.grid(row=1,column=0,columnspan=2)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backpage)).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).pack(side=tk.LEFT)



#! /usr/bin/env python3
import pickle, os, json, math, subprocess, numpy as np, pandas as pd, tkinter as tk
from tkinter import ttk
from tkinter import filedialog as fd

class ExperimentSetupStartPage(tk.Frame):

    def __init__(self, master, fName, bPage):
        global backPage
        global folderName
        folderName = fName
        backPage = bPage
        tk.Frame.__init__(self, master)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=(tk.TOP), padx=10, pady=10)
        tk.Label(mainWindow, text='Add markers (Input):').grid(row=2, column=0, sticky=(tk.W), pady=(10,
                                                                                                     0))
        modeVar = tk.StringVar(value='template')
        rb1 = tk.Radiobutton(mainWindow, text='Manually', padx=20, variable=modeVar, value='manual')
        rb2 = tk.Radiobutton(mainWindow, text='From template', padx=20, variable=modeVar, value='template')
        rb1.grid(row=3, column=0, sticky=(tk.W))
        rb2.grid(row=4, column=0, sticky=(tk.W))
        self.template = []

        def load_template():
            self.filename = fd.askopenfilename(filetypes=[('Excel Files', '*.xlsx'), ('CSV Files', '*.csv')])
            if self.filename != '':
                if '.csv' in self.filename:
                    self.template = pd.read_csv(self.filename)
                else:
                    self.template = pd.read_excel(self.filename)
                templateLabel['text'] = self.filename.split('/')[(-1)]
                templateLabel['font'] = 'Helvetica 14 bold'

        templateFrame = tk.Frame(mainWindow)
        templateFrame.grid(row=4, column=1, columnspan=2, sticky=(tk.W))
        templateSelectionButton = tk.Button(templateFrame, text='Browse:', command=(lambda : load_template()))
        templateSelectionButton.grid(row=0, column=0, sticky=(tk.W))
        templateLabel = tk.Label(templateFrame, text='')
        templateLabel.grid(row=0, column=1, sticky=(tk.W))

        def collectInput():
            method = modeVar.get()
            if method == 'manual':
                tk.messagebox.showinfo(title='Not supported', message='Option not currently supported. Please select a template file and try again.')
            else:
                for imageType in ('luminescent', 'brightfield'):
                    if imageType not in os.listdir('inputData/'):
                        os.mkdir('inputData/' + imageType)
                    for day in list(pd.unique(self.template['Day'])):
                        if day not in os.listdir('inputData/' + imageType):
                            os.mkdir('inputData/' + imageType + '/' + day)
                if 'templatePathDict.pkl' in os.listdir(master.homedirectory + 'misc'):
                    templatePathDict = pickle.load(open(master.homedirectory + 'misc/templatePathDict.pkl', 'rb'))
                else:
                    templatePathDict = {}
                projectName = os.getcwd().split('/')[(-2)]
                experimentName = os.getcwd().split('/')[(-1)]
                templatePathDict[projectName + '/' + experimentName] = self.filename
                with open(master.homedirectory + 'misc/templatePathDict.pkl', 'wb') as (f):
                    pickle.dump(templatePathDict, f)
                tk.messagebox.showinfo(title='Success', message='Experiment setup complete!')
                master.switch_frame(backPage, folderName)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=(tk.TOP), pady=10)
        tk.Button(buttonWindow, text='OK', command=(lambda : collectInput())).grid(row=5, column=0)
        tk.Button(buttonWindow, text='Back', command=(lambda : master.switch_frame(backPage, folderName))).grid(row=5, column=1)
        tk.Button(buttonWindow, text='Quit', command=quit).grid(row=5, column=2)



#!/usr/bin/env python3
import pickle,os,json
import numpy as np
import pandas as pd
import tkinter as tk
import tkinter.ttk
from radianceQuantifier.dataprocessing.inVivoRadianceProcessing import selectMatrices,plotMouseImages
import radianceQuantifier.dataprocessing.miscFunctions as mf
import radianceQuantifier.plotting.facetPlotLibrary as fpl 
import radianceQuantifier.plotting.interactiveGUIElements as ipe

expParamDict = {'radiance':'cell'}

#Get level names and values into an easily accessible dictionary
def createLabelDict(df):
    fulldf = df.stack()
    labelDict = {}
    for i in range(fulldf.index.nlevels):
        levelName = fulldf.index.levels[i].name
        if levelName not in ['Event','event']:
            labelDict[levelName] = list(pd.unique(fulldf.index.get_level_values(levelName)))
    return labelDict

def createLabelDictWithExperimentParameters(df,experimentParameters):
    fulldf = df.stack()
    labelDict = {}
    for i in range(fulldf.index.nlevels):
        levelName = fulldf.index.levels[i].name
        if levelName in ['Event','event']:
            pass
        else:
            if 'allLevelValues' in list(experimentParameters.keys()):
                experimentParameters['levelLabelDict'] = experimentParameters['allLevelValues']
            if levelName in experimentParameters['levelLabelDict'].keys():
                if len(experimentParameters['levelLabelDict'][levelName]) > 0:
                    labelDict[levelName] = experimentParameters['levelLabelDict'][levelName]
                else:
                    labelDict[levelName] = list(pd.unique(fulldf.index.get_level_values(levelName)))
            else:
                labelDict[levelName] = list(pd.unique(fulldf.index.get_level_values(levelName)))
    return labelDict

class checkUncheckAllButton(tk.Button):
    def __init__(self,parent,checkButtonList,**kwargs):
        tk.Button.__init__(self,parent,**kwargs)
        self.checkButtonList = checkButtonList
        self.parent = parent

    def checkAll(self):
        for checkButton in self.checkButtonList:
            checkButton.select()
    
    def uncheckAll(self):
        for checkButton in self.checkButtonList:
            checkButton.deselect()

class PlotExperimentWindow(tk.Frame):
    def __init__(self, master,fName,sPage):
        with open('misc/normalPlottingBool.pkl','wb') as f:
            pickle.dump(True,f)

        global folderName,switchPage
            
        folderName = fName
        switchPage = sPage

        tk.Frame.__init__(self, master)
        
        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        l2 = tk.Label(mainWindow, text="""Datatype: """)
        v2 = tk.StringVar(value='radiance')
        rb2a = tk.Radiobutton(mainWindow, text="Radiance",padx = 20, variable=v2, value='radiance')
        rb2b = tk.Radiobutton(mainWindow,text="Mouse Images",padx = 20, variable=v2, value='mouseImages')
        rb2c = tk.Radiobutton(mainWindow,text="Survival",padx = 20, variable=v2, value='survival')
        
        l2.grid(row=0,column=0)
        rb2a.grid(row=1,column=0,sticky=tk.W)
        rb2b.grid(row=2,column=0,sticky=tk.W)
        rb2c.grid(row=3,column=0,sticky=tk.W)
        
        def collectInputs():
            global useModifiedDf
            modified = False 
            useModifiedDf = modified
            if modified:
                modifiedString = '-modified'
            else:
                modifiedString = ''
            global dataType
            dataType = str(v2.get())
            global experimentDf
            experimentDf = pd.read_pickle('outputData/radianceStatisticPickleFile-'+folderName+modifiedString+'.pkl')
            if dataType == 'radiance':
                experimentDf = experimentDf.droplevel('Day')
                experimentDf.columns.name = 'Statistic'
                indexingDf = experimentDf.copy()
                experimentDf = experimentDf.unstack('Time').stack('Statistic')
                for i in range(len(experimentDf.index.names)-1,0,-1):
                    experimentDf = experimentDf.swaplevel(i,i-1)
                global trueLabelDict
                trueLabelDict = {}
                if experimentDf.columns.name == None:
                    experimentDf.columns.name = 'Marker'
                for level in indexingDf.index.names:
                    if level in experimentDf.index.names:
                        swapIndex = list(experimentDf.index.names).index(level)
                        correctOrder = indexingDf.index.unique(level).tolist()
                        experimentDf = experimentDf.swaplevel(0,swapIndex).reindex(labels=correctOrder,level=0,axis='index').swaplevel(0,swapIndex)

                experimentParametersBool = False
                for fn in os.listdir('misc'):
                    if 'experimentParameters' in fn:
                        if expParamDict[dataType] in fn:
                            experimentParametersBool = True
                            experimentParameters = json.load(open('misc/experimentParameters-'+folderName+'-'+expParamDict[dataType]+'.json','r'))
                if experimentParametersBool:
                    trueLabelDict = createLabelDictWithExperimentParameters(experimentDf,experimentParameters)
                else:
                    trueLabelDict = createLabelDict(experimentDf)
                master.switch_frame(PlotTypePage)
            elif dataType == 'mouseImages':
                global pixelMatrix,minScaleDict
                pixelMatrix = np.load('outputData/'+folderName+'-pixel.npz')
                minScaleDict = pickle.load(open('outputData/'+folderName+'-minScale.pkl','rb'))
                master.switch_frame(MouseImageSelectionPage)
            else:
                pass
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(switchPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class MouseImageSelectionPage(tk.Frame):
    def __init__(self, master):

        tk.Frame.__init__(self, master)
        tk.Label(self,text='Select days and groups to plot:',font='Helvetica 18 bold').pack(side=tk.TOP,padx=10)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=(10,0))

        #Get input df
        if 'templatePathDict.pkl' in os.listdir(master.homedirectory + 'misc'):
            templatePathDict = pickle.load(open(master.homedirectory + 'misc/templatePathDict.pkl', 'rb'))
        else:
            templatePathDict = {}
        projectName = os.getcwd().split('/')[(-2)]
        experimentName = os.getcwd().split('/')[(-1)]
        templatePath = templatePathDict[projectName + '/' + experimentName]
        if '.csv' in templatePath:
            sampleNameFile = pd.read_csv(templatePath)
        else:
            sampleNameFile = pd.read_excel(templatePath)
        
        days = pd.unique(sampleNameFile['Time']).tolist()
        groups = pd.unique(sampleNameFile['Group']).tolist()

        dayWindow = tk.Frame(mainWindow)
        contentWindow = tk.Frame(dayWindow)
        contentWindow.grid(row=2,column=0)
        dayWindow.grid(row=0,column=0,sticky=tk.N)
        tk.Label(dayWindow,text='Day:').grid(row=0,column=0,sticky=tk.EW)

        dayCBList,dayCBVarList = [],[]
        for i,day in enumerate(days):
            includeVar = tk.BooleanVar(value=True)
            cb = tk.Checkbutton(contentWindow,text=day,variable=includeVar)
            cb.grid(row=1+i,column=0,sticky=tk.W)
            dayCBList.append(cb)
            dayCBVarList.append(includeVar)
        
        checkButtonWindow = tk.Frame(dayWindow)
        checkAllButton1 = checkUncheckAllButton(checkButtonWindow,dayCBList, text='Check All')
        checkAllButton1.configure(command=checkAllButton1.checkAll)
        checkAllButton1.pack(side=tk.LEFT)
        uncheckAllButton1 = checkUncheckAllButton(checkButtonWindow,dayCBList, text='Uncheck All')
        uncheckAllButton1.configure(command=checkAllButton1.uncheckAll)
        uncheckAllButton1.pack(side=tk.LEFT)
        checkButtonWindow.grid(row=1,column=0,sticky=tk.EW)
        
        groupWindow = tk.Frame(mainWindow)
        groupWindow.grid(row=0,column=1,sticky=tk.N)
        contentWindow2 = tk.Frame(groupWindow)
        contentWindow2.grid(row=2,column=0)
        tk.Label(groupWindow,text='Group:').grid(row=0,column=0,sticky=tk.EW)
        groupCBList,groupCBVarList = [],[]
        #groupRenameEntryList = []
        for i,group in enumerate(groups):
            includeVar = tk.BooleanVar(value=True)
            cb = tk.Checkbutton(contentWindow2,text=group,variable=includeVar)
            cb.grid(row=1+i,column=0,sticky=tk.W)
            groupCBList.append(cb)
            groupCBVarList.append(includeVar)
            #e = tk.Entry(contentWindow2)
            #e.grid(row=1+i,column=1,sticky=tk.W)
            #e.insert(tk.END, '')
            #groupRenameEntryList.append(e)
        
        checkButtonWindow2 = tk.Frame(groupWindow)
        checkAllButton2 = checkUncheckAllButton(checkButtonWindow2,groupCBList, text='Check All')
        checkAllButton2.configure(command=checkAllButton2.checkAll)
        checkAllButton2.pack(side=tk.LEFT)
        uncheckAllButton2 = checkUncheckAllButton(checkButtonWindow2,groupCBList, text='Uncheck All')
        uncheckAllButton2.configure(command=checkAllButton2.uncheckAll)
        uncheckAllButton2.pack(side=tk.LEFT)
        checkButtonWindow2.grid(row=1,column=0,sticky=tk.EW)
        
        def collectInputs():
            global selectedDays,selectedGroups
            selectedDays = [days[i] for i,x in enumerate(dayCBVarList) if x.get()]
            selectedGroups = [groups[i] for i,x in enumerate(groupCBVarList) if x.get()]
            #groupRenamingDict = {groups[i]:x.get() for i,x in enumerate(groupRenameEntryList) if x.get() != ''}
            master.switch_frame(MouseGroupRenamingPage)
            #master.switch_frame(MouseImagePlottingOptionsPage)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(PlotExperimentWindow,folderName,switchPage)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class MouseGroupRenamingPage(tk.Frame):
    def __init__(self, master):

        tk.Frame.__init__(self, master)
        tk.Label(self,text='Rename groups (if needed):',font='Helvetica 18 bold').pack(side=tk.TOP,padx=10)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=(10,0))
        
        groupRenameEntryList = []
        for i,selectedGroup in enumerate(selectedGroups):
            tk.Label(mainWindow,text=selectedGroup+' -> ').grid(row=i,column=0,sticky=tk.W)
            e = tk.Entry(mainWindow)
            e.grid(row=i,column=1,sticky=tk.W)
            e.insert(tk.END, '')
            groupRenameEntryList.append(e)
        
        def collectInputs():
            global groupRenamingDict
            groupRenamingDict = {selectedGroups[i]:x.get() for i,x in enumerate(groupRenameEntryList) if x.get() != ''}
            master.switch_frame(MouseImagePlottingOptionsPage)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(MouseImageSelectionPage)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class MouseImagePlottingOptionsPage(tk.Frame):
    def __init__(self, master):

        tk.Frame.__init__(self, master)
        tk.Label(self,text='Plot aesthetics:',font='Helvetica 18 bold').pack(side=tk.TOP,padx=10)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=(10,0))
        
        #Select dataframe first to get selection title
        subsetMatrix,selectionKeysDf,selectionTitle = selectMatrices(pixelMatrix,days=selectedDays,groups=selectedGroups)

        tk.Label(mainWindow,text='Color map:').grid(row=0,column=0,sticky=tk.W)
        cmapEntry = tk.Entry(mainWindow)
        cmapEntry.grid(row=0,column=1,sticky=tk.W)
        cmapEntry.insert(tk.END, 'magma')

        tk.Label(mainWindow,text='Font size:').grid(row=1,column=0,sticky=tk.W)
        fontSizeEntry = tk.Entry(mainWindow)
        fontSizeEntry.grid(row=1,column=1,sticky=tk.W)
        fontSizeEntry.insert(tk.END, '42')
        
        tk.Label(mainWindow,text='Plot Title:').grid(row=2,column=0,sticky=tk.W)
        titleEntry = tk.Entry(mainWindow)
        titleEntry.grid(row=2,column=1,sticky=tk.W)
        titleEntry.insert(tk.END, selectionTitle)
        
        def collectInputs():
            plotMouseImages(subsetMatrix,minScaleDict,selectionKeysDf,innerCol='Sample',row='Day',col='Group',cmap=cmapEntry.get(),save_image=True,imageTitle=titleEntry.get(),fontsize=int(fontSizeEntry.get()),groupRenamingDict=groupRenamingDict)
            tk.messagebox.showinfo(title='Success', message='Plot created!')
            master.switch_frame(PlotExperimentWindow,folderName,switchPage)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(MouseGroupRenamingPage)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class PlotTypePage(tk.Frame):
    def __init__(self, master):

        if 'normalPlottingBool.pkl' in os.listdir('misc'):
            normalPlottingBool = pickle.load(open('misc/normalPlottingBool.pkl','rb'))
        else:
            normalPlottingBool = True

        if not normalPlottingBool:
            global useModifiedDf,dataType,experimentDf,trueLabelDict,folderName,switchPage
            plottingParams = pickle.load(open('misc/plottingParams.pkl','rb'))
            useModifiedDf = False
            dataType = 'cell'
            experimentDf = plottingParams['df']
            trueLabelDict = createLabelDict(experimentDf)
            folderName = plottingParams['folderName']
            switchPage = 'a' 

        plottableFigureDict = {'1d':['histogram','kde'],'categorical':['bar','violin','box','point','swarm','strip'],'2d':['line','scatter'],'3d':['heatmap']}
        
        tk.Frame.__init__(self, master)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10)
        
        l1 = tk.Label(mainWindow, text="What type of figure do you want to plot?",pady=10).grid(row=0,column = 0,columnspan = len(plottableFigureDict),sticky=tk.N)
        
        #global trueLabelDict
        #trueLabelDict = {}
        #trueLabelDict = createLabelDict(experimentDf)
         
        plotTypeRadioButtons = []
        plotSelectionString = tk.StringVar(value='1d/histogram')
        maxNumSubplots = 0
        for plotTypeIndex,plotTypeTitle in enumerate(plottableFigureDict):
            plotTypeTitleLabel = tk.Label(mainWindow,text=plotTypeTitle).grid(row=1,column=plotTypeIndex,sticky=tk.NW)
            temprblist = []
            tempselectionstring = []
            for subPlotTypeIndex,subPlotTitle in enumerate(plottableFigureDict[plotTypeTitle]):
                rb = tk.Radiobutton(mainWindow, text=subPlotTitle,padx = 20, variable=plotSelectionString, value=plotTypeTitle+'/'+subPlotTitle)
                rb.grid(row=subPlotTypeIndex+2,column=plotTypeIndex,sticky=tk.NW)
                temprblist.append(rb)
            plotTypeRadioButtons.append(temprblist)
            if len(plottableFigureDict[plotTypeTitle]) > maxNumSubplots:
                maxNumSubplots = len(plottableFigureDict[plotTypeTitle])
        
        stripSwarmBool = tk.BooleanVar()
        cb = tk.Checkbutton(mainWindow,text='Add strip/swarm points to categorical plot',variable=stripSwarmBool,pady=20)
        cb.grid(row=maxNumSubplots+2,column=0,columnspan=len(plottableFigureDict))
        
        def collectInputs():
            global plotType
            global subPlotType
            global addDistributionPoints
            addDistributionPoints = stripSwarmBool.get()
            plotType,subPlotType = plotSelectionString.get().split('/')
            master.switch_frame(selectLevelsPage,'a','b','c','d','e')
        
        def backCommand():
            #os.chdir('../../')
            if normalPlottingBool:
                master.switch_frame(PlotExperimentWindow,folderName,switchPage)
            else:
                master.switch_frame(plottingParams['homepage'],folderName,plottingParams['bp'],plottingParams['shp'])
            #master.switch_frame(PlotExperimentWindow,switchPage)
       
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP)
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Back",command=lambda: backCommand()).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).pack(side=tk.LEFT)

class selectLevelsPage(tk.Frame):
    def __init__(self, master,fsp,fName,backPage,pt,shp):
        tk.Frame.__init__(self, master)
        labelWindow = tk.Frame(self)
        labelWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        global figureLevelList,fullFigureLevelBooleanList
        fullFigureLevelBooleanList = []
        figureLevelList = []
        
        l1 = tk.Label(labelWindow, text="""Which levels names do you want to be included within this figure??:""").pack()
        mainWindow = tk.Frame(self)
        levelNameCheckButtons = []
        checkButtonVariableList = []
        for levelName,i in zip(trueLabelDict.keys(),range(len(trueLabelDict.keys()))):
            includeLevelBool = tk.BooleanVar(value=True)
            cb = tk.Checkbutton(mainWindow, text=levelName,padx = 20, variable=includeLevelBool,onvalue=True)
            cb.grid(row=i+3,column=1,sticky=tk.W)
            cb.select()
            levelNameCheckButtons.append(cb)
            checkButtonVariableList.append(includeLevelBool)
        
        checkButtonWindow = tk.Frame(self)
        checkAllButton1 = checkUncheckAllButton(checkButtonWindow,levelNameCheckButtons, text='Check All')
        checkAllButton1.configure(command=checkAllButton1.checkAll)
        checkAllButton1.pack(side=tk.LEFT)
        
        uncheckAllButton1 = checkUncheckAllButton(checkButtonWindow,levelNameCheckButtons, text='Uncheck All')
        uncheckAllButton1.configure(command=checkAllButton1.uncheckAll)
        uncheckAllButton1.pack(side=tk.LEFT)
        
        checkButtonWindow.pack(side=tk.TOP)
        mainWindow.pack(side=tk.TOP,padx=10)

        def collectInputs():
            includeLevelList = []
            for checkButtonVariable in checkButtonVariableList:
                includeLevelList.append(checkButtonVariable.get())
            for figureLevelBool,levelName in zip(includeLevelList,trueLabelDict):
                if figureLevelBool:
                    figureLevelList.append(levelName)
                    fullFigureLevelBooleanList.append(True)
                else:
                    fullFigureLevelBooleanList.append(False)
            master.switch_frame(selectLevelValuesPage,assignLevelsToParametersPage,trueLabelDict,selectLevelsPage,selectLevelsPage,fsp,fName,shp,'a')
        
        def quitCommand():
            exitBoolean = True
            quit()

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).pack(in_=buttonWindow,side=tk.LEFT)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(PlotTypePage)).pack(in_=buttonWindow,side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quitCommand()).pack(in_=buttonWindow,side=tk.LEFT)

class selectLevelValuesPage(tk.Frame):
    #(selectLevelValuesPage,SelectDimensionsPage,trueLabelDict,DataSelectionHomePage,finalSwitchPage,folderName,secondaryhomepage,processType)
    #master.switch_frame(selectLevelValuesPage,SelectDimensionsPage,trueLabelDict,DataSelectionHomePage,backpage,finalSwitchPage,folderName,secondaryhomepage,processType)
    def __init__(self, master,switchPage,trueLabelDict,backPage,secondaryBackPage,fsp,fName,shp,pt):
        tk.Frame.__init__(self, master)
        
        includeLevelValueList = []
        
        labelWindow = tk.Frame(self)
        labelWindow.pack(side=tk.TOP,padx=10,fill=tk.X,expand=True)
        
        l1 = tk.Label(labelWindow, text='Which specific level values do you want to include in the figure?',pady=10).grid(row=0,column = 0,columnspan=len(trueLabelDict)*6)
        levelValueCheckButtonList = []
        overallCheckButtonVariableList = []
        checkAllButtonList = []
        uncheckAllButtonList = []
        i=0
        maxNumLevelValues = 0
        for levelName in trueLabelDict:
            if len(trueLabelDict[levelName]) > maxNumLevelValues:
                maxNumLevelValues = len(trueLabelDict[levelName])
        """BEGIN TEMP SCROLLBAR CODE"""
        labelWindow1 = tk.Frame(self)
        labelWindow1.pack(side=tk.TOP,padx=10,fill=tk.X,expand=True)
        
        #Make canvas
        w1 = tk.Canvas(labelWindow1, width=1200, height=400, scrollregion=(0,0,2000,33*maxNumLevelValues))

        #Make scrollbar
        scr_v1 = tk.Scrollbar(labelWindow1,orient=tk.VERTICAL)
        scr_v1.pack(side=tk.RIGHT,fill=tk.Y)
        scr_v1.config(command=w1.yview)
        #Add scrollbar to canvas
        w1.config(yscrollcommand=scr_v1.set)
        
        scr_v2 = tk.Scrollbar(labelWindow1,orient=tk.HORIZONTAL)
        scr_v2.pack(side=tk.BOTTOM,fill=tk.X)
        scr_v2.config(command=w1.xview)
        w1.config(xscrollcommand=scr_v2.set)
        w1.pack(fill=tk.BOTH,expand=True)
        #Make and add frame for widgets inside of canvas
        #canvas_frame = tk.Frame(w1)
        labelWindow = tk.Frame(w1)
        labelWindow.pack()
        w1.create_window((0,0),window=labelWindow, anchor = tk.NW)
        """END TEMP SCROLLBAR CODE"""
        for levelName in trueLabelDict:
            j=0
            levelCheckButtonList = []
            levelCheckButtonVariableList = []
            levelLabel = tk.Label(labelWindow, text=levelName+':')
            levelLabel.grid(row=1,column = i*6,sticky=tk.N,columnspan=5)
            for levelValue in trueLabelDict[levelName]:
                includeLevelValueBool = tk.BooleanVar()
                cb = tk.Checkbutton(labelWindow, text=levelValue, variable=includeLevelValueBool)
                cb.grid(row=j+4,column=i*6+2,columnspan=2,sticky=tk.W)
                labelWindow.grid_columnconfigure(i*6+3,weight=1)
                cb.select()
                levelCheckButtonList.append(cb)
                levelCheckButtonVariableList.append(includeLevelValueBool)
                j+=1
            
            checkAllButton1 = checkUncheckAllButton(labelWindow,levelCheckButtonList, text='Check All')
            checkAllButton1.configure(command=checkAllButton1.checkAll)
            checkAllButton1.grid(row=2,column=i*6,sticky=tk.N,columnspan=3)
            checkAllButtonList.append(checkAllButton1)
            
            uncheckAllButton1 = checkUncheckAllButton(labelWindow,levelCheckButtonList, text='Uncheck All')
            uncheckAllButton1.configure(command=checkAllButton1.uncheckAll)
            uncheckAllButton1.grid(row=2,column=i*6+3,sticky=tk.N,columnspan=3)
            uncheckAllButtonList.append(checkAllButton1)

            levelValueCheckButtonList.append(levelCheckButtonList)
            overallCheckButtonVariableList.append(levelCheckButtonVariableList)
            i+=1

        def collectInputs():
            for checkButtonVariableList in overallCheckButtonVariableList:
                tempLevelValueList = []
                for checkButtonVariable in checkButtonVariableList:
                    tempLevelValueList.append(checkButtonVariable.get())
                includeLevelValueList.append(tempLevelValueList)
            #master.switch_frame(assignLevelsToParametersPage)
            master.switch_frame(switchPage,includeLevelValueList)
        
        def quitCommand():
            exitBoolean = True
            quit()
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=maxNumLevelValues+4,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backPage,fsp,fName,secondaryBackPage,pt,shp)).grid(row=maxNumLevelValues+4,column=1)
        tk.Button(buttonWindow, text="Quit",command=lambda: quitCommand()).grid(row=maxNumLevelValues+4,column=2)

class assignLevelsToParametersPage(tk.Frame):
    
    def __init__(self, master,temp):
        parameterTypeDict = {
                'categorical':['Color','Order', 'Row', 'Column','None'],
                '1d':['Color','Row','Column','None'],
                '2d':['Marker','Color','Size','Row','Column','X Axis Values','None'],
                '3d':['Row','Column','X Axis Values','Y Axis Values']}
        
        tk.Frame.__init__(self, master)
        global includeLevelValueList
        includeLevelValueList = temp
        global parametersSelected
        parametersSelected = {}
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        l1 = tk.Label(mainWindow, text='Which plotting parameter do you want to assign to each of your figure levels?',pady=10).grid(row=0,column = 0,columnspan = len(figureLevelList))
        rblist = []
        parameterVarList = []
        for figureLevel,figureLevelIndex in zip(figureLevelList,range(len(figureLevelList))):
            v = tk.IntVar()
            temprblist = []
            levelLabel = tk.Label(mainWindow, text=figureLevel+':')
            levelLabel.grid(row=1,column=figureLevelIndex,sticky=tk.NW)
            for plottingParameter,parameterIndex in zip(parameterTypeDict[plotType],range(len(parameterTypeDict[plotType]))):
                rb = tk.Radiobutton(mainWindow, text=plottingParameter,padx = 20, variable=v, value=parameterIndex)
                rb.grid(row=parameterIndex+2,column=figureLevelIndex,sticky=tk.NW)
                temprblist.append(rb)
            rblist.append(temprblist)
            parameterVarList.append(v)
        
        def collectInputs():
            for parameterVar,levelName in zip(parameterVarList,figureLevelList):
                if parameterTypeDict[plotType][parameterVar.get()] not in parametersSelected.keys():
                    parametersSelected[parameterTypeDict[plotType][parameterVar.get()]] = levelName
                else:
                    if not isinstance(parametersSelected[parameterTypeDict[plotType][parameterVar.get()]],list):
                        parametersSelected[parameterTypeDict[plotType][parameterVar.get()]] = [parametersSelected[parameterTypeDict[plotType][parameterVar.get()]]]+[levelName]
                    else:
                        parametersSelected[parameterTypeDict[plotType][parameterVar.get()]].append(levelName)
            master.switch_frame(plotElementsGUIPage)
        
        def quitCommand():
            exitBoolean = True
            quit()
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=len(parameterTypeDict[plotType])+2,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(selectLevelValuesPage,assignLevelsToParametersPage,trueLabelDict,selectLevelsPage,'a','b','c','d','e')).grid(row=len(parameterTypeDict[plotType])+2,column=1)
        tk.Button(buttonWindow, text="Quit",command=lambda: quitCommand()).grid(row=len(parameterTypeDict[plotType])+2,column=2)

def getDefaultAxisTitles():
        xaxistitle = ''
        yaxistitle = ''
        cbartitle = ''
        cytokineDefault = 'Concentration (nM)'
        if plotType == '1d':
            xaxistitle = 'MFI'
            yaxistitle = 'Frequency'
        else:
            if plotType == 'categorical':
                xaxistitle = parametersSelected['Order']
                if dataType == 'cyt':
                    yaxistitle = cytokineDefault
            else:
                xaxistitle = parametersSelected['X Axis Values']
                if dataType == 'cyt':
                    if plotType == '2d':
                        yaxistitle = cytokineDefault
                    else:
                        cbartitle = cytokineDefault
        if xaxistitle == 'Time':
            xaxistitle += ' (days)'
        return [xaxistitle,yaxistitle,cbartitle]

class plotElementsGUIPage(tk.Frame):
    def __init__(self, master):
        if 'experimentParameters-'+folderName+'-'+expParamDict[dataType]+'.json' in os.listdir('misc'):
            experimentParameters = json.load(open('misc/experimentParameters-'+folderName+'-'+expParamDict[dataType]+'.json','r'))
        else:
            tempDict = {}
            stackedDf = experimentDf.stack()
            for level in stackedDf.index.names:
                levelValues = list(pd.unique(stackedDf.index.get_level_values(level)))
                tempDict[level] = levelValues
            experimentParameters = {}
            experimentParameters['levelLabelDict'] = tempDict
        """ 
        global dataType
        if 'cell' in pickleFileName: 
            dataType = 'cell'
        elif 'cyt' in pickleFileName:
            dataType = 'cyt'
        elif 'prolif' in pickleFileName:
            dataType = 'prolif'
        else:
            dataType = ''
        """
        self.tld = trueLabelDict

        axisDict = {'categorical':['X','Y'],'1d':['Y'],'2d':['X','Y'],'3d':['X','Y','Colorbar']}
        scalingList = ['Linear','Logarithmic','Biexponential']
        axisSharingList = ['col','row','']
        axisTitleDefaults = getDefaultAxisTitles() 
        
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)

        tk.Label(mainWindow, text='Title: ').grid(row=1,column=0,sticky=tk.W)
        for scaling,scalingIndex in zip(scalingList,range(len(scalingList))):
            tk.Label(mainWindow, text=scaling+' Scaling: ').grid(row=scalingIndex+2,column=0,sticky=tk.W)
        tk.Label(mainWindow, text='Linear Range (Biexponential Scaling): ').grid(row=len(scalingList)+2,column=0,sticky=tk.W)
        tk.Label(mainWindow, text='Convert to numeric: ').grid(row=len(scalingList)+3,column=0,sticky=tk.W)
        tk.Label(mainWindow, text='Share axis across: ').grid(row=len(scalingList)+4,column=0,sticky=tk.W)
        tk.Label(mainWindow, text='Axis limits: ').grid(row=len(scalingList)+5,column=0,sticky=tk.W)

        entryList = []
        scalingVariableList = []
        radioButtonList = []
        checkButtonList = []
        checkButtonVarList = []
        radioButtonList2 = []
        radioButtonVarList2 = []
        linearRangeScalingList = []
        limitEntryList = []
        for axis,axisIndex in zip(axisDict[plotType],range(len(axisDict[plotType]))):
            tk.Label(mainWindow, text=axis+ ' Axis').grid(row=0,column=axisIndex+1)
            
            e1 = tk.Entry(mainWindow)
            e1.grid(row=1,column=axisIndex+1)
            e1.insert(0, axisTitleDefaults[axisIndex])
            entryList.append(e1)
            
            axisRadioButtonList = []
            v = tk.StringVar(value='Linear')
            for scaling,scalingIndex in zip(scalingList,range(len(scalingList))):
                rb = tk.Radiobutton(mainWindow,variable=v,value=scaling)
                rb.grid(row=scalingIndex+2,column=axisIndex+1)
                axisRadioButtonList.append(rb)
            radioButtonList.append(axisRadioButtonList)
            scalingVariableList.append(v)
            
            e2 = tk.Entry(mainWindow)
            e2.grid(row=len(scalingList)+2,column=axisIndex+1)
            linearRangeScalingList.append(e2)
            
            b = tk.BooleanVar(value=False)
            cb = tk.Checkbutton(mainWindow,variable=b)
            cb.grid(row=len(scalingList)+3,column=axisIndex+1)
            checkButtonList.append(cb)
            checkButtonVarList.append(b)
       
            shareWindow = tk.Frame(mainWindow)
            shareWindow.grid(row=len(scalingList)+4,column=axisIndex+1)
            shareString = tk.StringVar(value='None')
            rb2a = tk.Radiobutton(shareWindow,variable=shareString,text='All',value='all')
            rb2b = tk.Radiobutton(shareWindow,variable=shareString,text='Row',value='row')
            rb2c = tk.Radiobutton(shareWindow,variable=shareString,text='Col',value='col')
            rb2d = tk.Radiobutton(shareWindow,variable=shareString,text='None',value='none')
            shareString.set('all')
            rb2a.grid(row=0,column=1)
            rb2b.grid(row=0,column=2)
            rb2c.grid(row=0,column=3)
            rb2d.grid(row=0,column=4)
            radioButtonList2.append([rb2a,rb2b])
            radioButtonVarList2.append(shareString)

            limitWindow = tk.Frame(mainWindow)
            limitWindow.grid(row=len(scalingList)+5,column=axisIndex+1)
            #ll = tk.Label(limitWindow,text='Lower:').grid(row=0,column=0)
            e3 = tk.Entry(limitWindow,width=5)
            e3.grid(row=0,column=1)
            #ul = tk.Label(limitWindow,text='Upper:').grid(row=0,column=2)
            e4 = tk.Entry(limitWindow,width=5)
            e4.grid(row=0,column=3)
            limitEntryList.append([e3,e4])

        def collectInputs(plotAllVar):
            plotOptions = {}
            for axis,axisIndex in zip(axisDict[plotType],range(len(axisDict[plotType]))):
                share = radioButtonVarList2[axisIndex].get()
                if share == 'none':
                    share = False
                plotOptions[axis] = {'axisTitle':entryList[axisIndex].get(),
                        'axisScaling':scalingVariableList[axisIndex].get(),
                        'linThreshold':linearRangeScalingList[axisIndex].get(),
                        'numeric':checkButtonVarList[axisIndex].get(),
                        'share':share,
                        'limit':[limitEntryList[axisIndex][0].get(),limitEntryList[axisIndex][1].get()]}
            
            plotSpecificDict = {}
            if subPlotType == 'kde':
                scaleBool = ipe.getRadiobuttonValues(modeScaleRadiobuttonVarsDict)['scale to mode']
                if scaleBool == 'yes':
                    plotSpecificDict['scaleToMode'] = True
                else:
                    plotSpecificDict['scaleToMode'] = False
                plotSpecificDict['smoothing'] = int(ipe.getSliderValues(smoothingSliderList,['smoothing'])['smoothing'])
            
            useModifiedDf = False
            sName = titleEntry.get()
            subsettedDfList,subsettedDfListTitles,figureLevels,levelValuesPlottedIndividually = fpl.produceSubsettedDataFrames(experimentDf.stack().to_frame('temp'),fullFigureLevelBooleanList,includeLevelValueList,self.tld)
            fpl.plotFacetedFigures(folderName,plotType,subPlotType,dataType,subsettedDfList,subsettedDfListTitles,figureLevels,levelValuesPlottedIndividually,useModifiedDf,experimentDf,plotOptions,parametersSelected,addDistributionPoints,originalLevelValueOrders=experimentParameters['levelLabelDict'],subfolderName=sName,context=ipe.getRadiobuttonValues(contextRadiobuttonVarsDict)['context'],height=float(heightEntry.get()),aspect=float(widthEntry.get()),titleBool=ipe.getRadiobuttonValues(plotTitleRadiobuttonVarsDict)['plotTitle'],colwrap=int(colWrapEntry.get()),legendBool=ipe.getRadiobuttonValues(legendRadiobuttonVarsDict)['legend'],cmap=cmapEntry.get(),plotAllVar=plotAllVar,titleAdjust=titleAdjustEntry.get(),plotSpecificDict = plotSpecificDict)
            
        titleWindow = tk.Frame(self)
        titleWindow.pack(side=tk.TOP,pady=10)
        tk.Label(titleWindow,text='Enter subfolder for these plots (optional):').grid(row=0,column=0)
        titleEntry = tk.Entry(titleWindow,width=15)
        titleEntry.grid(row=0,column=1)
        
        miscOptionsWindow = tk.Frame(self)
        miscOptionsWindow.pack(side=tk.TOP,pady=10)
        
        contextWindow = tk.Frame(miscOptionsWindow)
        contextWindow.grid(row=0,column=0,sticky=tk.N)
        contextRadiobuttonList,contextRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(contextWindow,['context'],{'context':['notebook','talk','poster']}) 
        
        figureDimensionWindow = tk.Frame(miscOptionsWindow)
        figureDimensionWindow.grid(row=0,column=1,sticky=tk.N)
        tk.Label(figureDimensionWindow,text='figure dimensions').grid(row=0,column=0)
        tk.Label(figureDimensionWindow,text='height:').grid(row=1,column=0)
        tk.Label(figureDimensionWindow,text='width:').grid(row=2,column=0)
        heightEntry = tk.Entry(figureDimensionWindow,width=3)
        if plotType != '1d':
            heightEntry.insert(0, '5')
        else:
            heightEntry.insert(0, '3')
        widthEntry = tk.Entry(figureDimensionWindow,width=3)
        widthEntry.insert(0, '1')
        heightEntry.grid(row=1,column=1)
        widthEntry.grid(row=2,column=1)
        
        plotTitleWindow = tk.Frame(miscOptionsWindow)
        plotTitleWindow.grid(row=0,column=2,sticky=tk.N)
        plotTitleRadiobuttonList,plotTitleRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(plotTitleWindow,['plotTitle'],{'plotTitle':['yes','no']}) 
        
        legendWindow = tk.Frame(miscOptionsWindow)
        legendWindow.grid(row=0,column=3,sticky=tk.N)
        legendRadiobuttonList,legendRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(legendWindow,['legend'],{'legend':['yes','no']}) 
        
        colWrapWindow = tk.Frame(miscOptionsWindow)
        colWrapWindow.grid(row=0,column=4,sticky=tk.N)
        tk.Label(colWrapWindow,text='column wrap:').grid(row=0,column=0)
        colWrapEntry = tk.Entry(colWrapWindow,width=5)
        colWrapEntry.insert(0, '5')
        colWrapEntry.grid(row=1,column=0)
        
        titleAdjustWindow = tk.Frame(miscOptionsWindow)
        titleAdjustWindow.grid(row=0,column=5,sticky=tk.N)
        tk.Label(titleAdjustWindow,text='title location (% of window):').grid(row=0,column=0)
        titleAdjustEntry = tk.Entry(titleAdjustWindow,width=5)
        titleAdjustEntry.insert(0, '')
        titleAdjustEntry.grid(row=1,column=0)
        
        #outlierWindow = tk.Frame(miscOptionsWindow)
        #outlierWindow.grid(row=0,column=6,sticky=tk.N)
        #outlierRadiobuttonList,outlierRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(outlierWindow,['remove outliers'],{'remove outliers':['yes','no']}) 
        #outlierRadiobuttonVarsDict['remove outliers'].set('no')
        
        cmapWindow = tk.Frame(miscOptionsWindow)
        cmapWindow.grid(row=0,column=6,sticky=tk.N)
        tk.Label(cmapWindow,text='Colormap:').grid(row=0,column=0)
        cmapEntry = tk.Entry(cmapWindow,width=10)
        cmapEntry.grid(row=1,column=0)
        
        if subPlotType == 'kde':
            #Scale to mode button
            subPlotSpecificWindow = tk.Frame(self)
            subPlotSpecificWindow.pack(side=tk.TOP,pady=10)
            modeScaleWindow = tk.Frame(subPlotSpecificWindow)
            modeScaleWindow.grid(row=0,column=0,sticky=tk.N)
            modeScaleRadiobuttonList,modeScaleRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(modeScaleWindow,['scale to mode'],{'scale to mode':['yes','no']}) 
            modeScaleRadiobuttonVarsDict['scale to mode'].set('no')
            #Smoothness (or bandwidth) slider
            smoothnessWindow = tk.Frame(subPlotSpecificWindow)
            smoothnessWindow.grid(row=0,column=1,sticky=tk.N)
            smoothingSliderList = ipe.createParameterAdjustmentSliders(smoothnessWindow,['smoothing'],{'smoothing':[1,99,2,27]})
        
        plotButtonWindow = tk.Frame(self)
        plotButtonWindow.pack(side=tk.TOP,pady=10)
        tk.Button(plotButtonWindow, text="Generate First Plot",command=lambda: collectInputs(False)).grid(row=0,column=0)
        tk.Button(plotButtonWindow, text="Generate All Plots",command=lambda: collectInputs(True)).grid(row=0,column=1)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        def okCommand():
            master.switch_frame(PlotTypePage)
        
        tk.Button(buttonWindow, text="Finish",command=lambda: okCommand()).grid(row=len(scalingList)+4,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(assignLevelsToParametersPage,includeLevelValueList)).grid(row=len(scalingList)+4,column=1)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).grid(row=len(scalingList)+4,column=2)
